// clasmapi.cpp : Defines the class behaviors for CMAPI, CMapiRecip, & CMapiReturnRecip.
//
/////////////////////////////////////////////////////////////////////////////
//  The MAPI classes are designed to provide an MFC shell around the MAPI
//  functions released with MS:Mail 3.x
//	
//	If anyone has questions  or improvements to the classes please let 
//	me know at my CompuServe account.
//  
//  Written by Tony Nathan.  30/7/'94	
//  CompuServe account No. 70302,1332
//
#include "stdafx.h"

#include "clasmapi.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC( CMapiException, CException )

static char BASED_CODE szErrFmt[] = "MAPI Error: %li";
CMapiException::CMapiException(DWORD dwEc)
{
  	m_dwErrorCode = dwEc;
  	if(m_dwErrorCode > SUCCESS_SUCCESS && m_dwErrorCode <= (DWORD)MAPI_E_NO_RECIP)
  	{
  		VERIFY(m_strError.LoadString((int)m_dwErrorCode));
  	}
  	else
  	{
  		wsprintf(m_strError.GetBuffer(60), szErrFmt, m_dwErrorCode);
  		m_strError.ReleaseBuffer();  		
  	}
}

CMapiException::~CMapiException()
{
	m_strError.Empty();
}

void AFXAPI ThrowMapiException(DWORD dwError)
{
	CMapiException* e = new CMapiException(dwError);
	THROW(e);
}

IMPLEMENT_SERIAL( CMapiRecip, CObject, 0 )

////////////////////////////////////////////////////////////////////////
// Creates class and sets members to NULL
//
CMapiRecip::CMapiRecip()
{ 	
	ASSERT_VALID( this );
	m_pMapiRecipDesc = NULL;
	m_pPosition = NULL;
	m_ulRecipCount = 0;
}

//////////////////////////////////////////////////////////////////////////
// Copy  constructor
CMapiRecip::CMapiRecip( CMapiRecip& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	
	TRY
	{
		*this = b;
	}
	CATCH( CException, e )
	{
		THROW_LAST();
		m_pMapiRecipDesc = NULL;
		m_pPosition = NULL;
		m_ulRecipCount = 0;
	}
	END_CATCH
}


////////////////////////////////////////////////////////////////////
// allows the building of an object from a pointer.
// dwCount is the number of elements in the array.
//
CMapiRecip::CMapiRecip( lpMapiRecipDesc pMapiRecip, DWORD dwCount )
{
	ASSERT_VALID( this );
	DWORD err;
	
	if ( ( err = CopyFromPointer( pMapiRecip, dwCount ) ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);		
	}	
}

//////////////////////////////////////////////////////////////
//	CMapiRecip::~CMapiRecip
//
//	Deletes all memory assoicated with the object.
//
CMapiRecip::~CMapiRecip()
{
	ASSERT_VALID( this );
	Flush();
}

///////////////////////////////////////////////////////////////////////
//	CMapiRecip::Flush()
//  
//  cleans out the all memory allacated.
//  Calle in distructor
//
void CMapiRecip::Flush()
{
	ASSERT_VALID( this );
	if ( m_pMapiRecipDesc != NULL )
	{
		m_pPosition = m_pMapiRecipDesc;
		
		ULONG i;
		for ( i=0 ; i<m_ulRecipCount ; ++i )
		{
			delete m_pPosition->lpszName;
			m_pPosition->lpszName = NULL;

			delete m_pPosition->lpszAddress;
			m_pPosition->lpszAddress = NULL;

			delete m_pPosition->lpEntryID;
			m_pPosition->lpEntryID = NULL;

			++m_pPosition;
		}
		delete m_pMapiRecipDesc;
		m_pMapiRecipDesc = m_pPosition = NULL;
	}
	m_ulRecipCount = 0;
}

/////////////////////////////////////////////////////////////////////
//	CMapiRecipDesc::CreatePointer
//	allocates memory for array of MapiRecipDesc structures of dwSize
//  and sets contents to NULL
//
//	returns SUCCESS_SUCCESS, POINTER_ALREADY_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiRecip::CreatePointer( WORD wSize )
{
	ASSERT_VALID( this );
	
	if ( wSize > 0)
	{
		if ( m_pMapiRecipDesc == NULL )
		{
		m_pMapiRecipDesc = new MapiRecipDesc[wSize];
			if ( m_pMapiRecipDesc == NULL )
			{
				return( MEMORY_ERROR_ON_NEW );
			}
			m_ulRecipCount = (ULONG)wSize;
			
			_fmemset( m_pMapiRecipDesc, NULL, ( sizeof(MapiRecipDesc) * wSize ) );
			m_pPosition = m_pMapiRecipDesc;
		}
		else
		{
			return( POINTER_ALREADY_THERE );		
		}
		
	}
	return( SUCCESS_SUCCESS );
}

//////////////////////////////////////////////
// Gets a pointer to the structure. 
// private function( is used by friends CMapiMessage & CMAPI )
//
// returns a pointer or NULL
//		
inline lpMapiRecipDesc CMapiRecip::GetPointer()
{	
	ASSERT_VALID( this );
	return ( m_pMapiRecipDesc );
}
		
///////////////////////////////////////////////
// Gets the size of array of structures or the 
// number of recipents in the object.
//
DWORD CMapiRecip::GetSize()
{	
	ASSERT_VALID( this );
	return ( (DWORD)m_ulRecipCount );
} 

////////////////////////////////////////////////////
// Gets the CLASS of the mail message at the current position
// eg:- MAPI_ORIG, MAPI_TO, MAPI_CC, MAPI_BCC
//
inline DWORD CMapiRecip::GetRecipClass()
{	
	ASSERT_VALID( this );
	ASSERT( m_pMapiRecipDesc != NULL );
	return ( (DWORD)m_pPosition->ulRecipClass );
}
		
//////////////////////////////////////////////////////
// Sets the class off the recipent. 
// eg:- MAPI_ORIG, MAPI_TO, MAPI_CC, MAPI_BCC
//
// returns SUCCESS_SUCCESS or POINTER_NOT_THERE
//		
DWORD CMapiRecip::SetRecipClass( DWORD dwNewRecipClass )
{	
	ASSERT_VALID( this );
	if ( m_pMapiRecipDesc == NULL )
	{
		return ( POINTER_NOT_THERE );
	}
	
	m_pPosition->ulRecipClass = (ULONG)dwNewRecipClass;
	return ( SUCCESS_SUCCESS );
}

////////////////////////////////////////////////////////
// Gets the name from a MapiRecipDesc struct as a
// CString object.
//
// Throws a CMapiException if there is an error.
// 
CString CMapiRecip::GetName() 
{	
	if ( m_pMapiRecipDesc == NULL )
	{
		ThrowMapiException(POINTER_NOT_THERE);		
		m_stringRetval = "";
		m_stringRetval.Empty();
		return( m_stringRetval );
	}
	m_stringRetval = m_pPosition->lpszName;

	return ( m_stringRetval );
}

////////////////////////////////////////////////////////
// Gets the address from a MapiRecipDesc struct as a
// CString object.
//
// Throws a CMapiException if there is an error.
// 
CString CMapiRecip::GetAddress() 
{
	if ( m_pMapiRecipDesc == NULL )
	{
		ThrowMapiException(POINTER_NOT_THERE);		
					
		m_stringRetval = "";
		return( m_stringRetval );
	}
	m_stringRetval = m_pPosition->lpszAddress;

	return ( m_stringRetval );
}


/////////////////////////////////////////////////////////////////////
//	CMapiRecipDesc::SetPosition
//	sets the postion for all other member functions	by the use of an
//  internal pointer.
//
//	returns SUCCESS_SUCCESS, POINTER_NOT_THERE
//
DWORD CMapiRecip::SetPosition( DWORD dwPos )
{	
	ASSERT_VALID( this );
	ASSERT( dwPos <= m_ulRecipCount );

	if ( m_ulRecipCount == 0 )
	{
		return( POINTER_NOT_THERE );
		
	}
	// re-initialise the pointer m_pPosition
	m_pPosition = m_pMapiRecipDesc;
	
	// set new position
	m_pPosition += dwPos;
	
	return( SUCCESS_SUCCESS );
}
	
/////////////////////////////////////////////////////////////
//	CMapiRecip::SetName
//
//	Sets the name in a MapiRecipDesc structure;
//  returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or MEMORY_ERROR_ON_NEW.
//
DWORD CMapiRecip::SetName( CString NewName )
{	
	ASSERT_VALID( this );
	if ( m_pMapiRecipDesc == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// delete old string if there
	delete m_pPosition->lpszName;
	m_pPosition->lpszName = NULL;
	
	if ( NewName.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}
	
	m_pPosition->lpszName = (LPSTR) new char[NewName.GetLength() + 1]; 
	if ( m_pPosition->lpszName == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pPosition->lpszName, NewName );
	NewName.ReleaseBuffer();
	return ( SUCCESS_SUCCESS );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiRecip::SetAddress
//
//	Sets the address in a MapiRecipDesc structure.
//
//	returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiRecip::SetAddress( CString NewAddress )
{	ASSERT_VALID( this );
	if ( m_pMapiRecipDesc == NULL )
	{
		return( POINTER_NOT_THERE );		
	}

	// delete old string if there
	delete m_pPosition->lpszAddress;
	m_pPosition->lpszAddress = NULL;
	

	if ( NewAddress.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pPosition->lpszAddress = (LPSTR)new char[NewAddress.GetLength() + 1];
	if ( m_pPosition == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pPosition->lpszAddress, NewAddress );
	NewAddress.ReleaseBuffer();
	return ( SUCCESS_SUCCESS );
}			


//////////////////////////////////////////////////////////////
//	CMapiRecip::SetEntryID
//	copies the contents of pEntryID into this CMapiRecip object
//
//	returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiRecip::SetEntryID ( LPVOID pEntryID, WORD wEIDSize )
{
	if ( m_pMapiRecipDesc == NULL )
	{
		return( POINTER_NOT_THERE );		
	}

	delete m_pPosition->lpEntryID;
	
	m_pPosition->ulEIDSize = (ULONG) wEIDSize;
	if ( wEIDSize == 0 )
	{
		return( SUCCESS_SUCCESS );		
	}
	m_pPosition->lpEntryID = (LPVOID)new BYTE[wEIDSize];
	
	LPVOID pdummy = m_pPosition->lpEntryID;

	if ( m_pPosition->lpEntryID == NULL )
	{
		return( MEMORY_ERROR_ON_NEW );		
	}
	memcpy( pdummy, (LPVOID)pEntryID, wEIDSize );
	return( SUCCESS_SUCCESS );
}
	
///////////////////////////////////////////////////////////////////////
//  CMapiRecip::CopyFromPointer( MapiRecipDesc* pMapiRecip, DWORD dwCount )
//  
//  private function only also used by friends CMAPI and CMapiRecip
//
//	initialises the class with a pointer returned by a mapi function.
//	For use inside a CMapiMessage object or a CMAPI object.
//
//	returns SUCCESS_SUCCESS, POINTER_ALREADY_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiRecip::CopyFromPointer( lpMapiRecipDesc pMapiRecip, DWORD dwCount )
{
	ASSERT_VALID( this );
	DWORD retval, i, r;
	lpMapiRecipDesc pTemp = pMapiRecip;
	
	Flush();
	
	if ( pMapiRecip == NULL || dwCount == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}
	
	retval = CreatePointer( (WORD)dwCount );
	if ( retval != SUCCESS_SUCCESS )
	{
		return( retval );		
	}
	
	for ( i=0 ; i<dwCount ; ++i )
	{
		SetPosition( i );
		
		SetRecipClass( (DWORD)pTemp->ulRecipClass );
		
		retval = SetName( pTemp->lpszName );
		if ( retval != SUCCESS_SUCCESS )
		{
			//	Clean up all creations to ensure free memory when an error occurs.
			if ( i > 0 )
			{
				for ( r=0 ; r<i ; ++r )
				{
					SetPosition( r );
					SetName( "" );
					SetAddress( "" );
				}
			}
			return ( retval );	
		}
	
		retval = SetAddress( pTemp->lpszAddress );
		if ( retval != SUCCESS_SUCCESS )
		{
			//	Clean up all creations to ensure free memory when an error occurs.
			SetName( "" );
			if ( i > 0 )
			{
				for ( r=0 ; r<i ; ++r )
				{
					SetPosition( r );
					SetName( "" );
					SetAddress( "" );
				}
			}
			return ( retval );	
		}
		retval = SetEntryID( pTemp->lpEntryID, (WORD)pTemp->ulEIDSize );
		if ( retval != SUCCESS_SUCCESS )
		{
			//	Clean up all creations to ensure free memory when an error occurs.
			SetName( "" );
			SetAddress( "" );
			if ( i > 0 )
			{
				for ( r=0 ; r<i ; ++r )
				{
					SetPosition( r );
					SetName( "" );
					SetAddress( "" );
				}
			}
			return ( retval );	
		}
		m_pPosition->ulReserved = pTemp->ulReserved;
		m_pPosition->ulRecipClass = pTemp->ulRecipClass;
		++pTemp;
	}
	return( SUCCESS_SUCCESS );
}

////////////////////////////////////////////////////////
// does the same as SetPosition 
//
// can through a CMapiException
//
CMapiRecip& CMapiRecip::operator[]( DWORD d )
{
	DWORD err;
	ASSERT( d <= m_ulRecipCount );
	
	if ( ( err = SetPosition( d ) ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);
	}
	
	return ( *this );
}

/////////////////////////////////////////////////////////////////
// equals operator will does the same as copy constructor
// can throw a CMapiException
// 
CMapiRecip& CMapiRecip::operator=(CMapiRecip& b)
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	DWORD err;
	
	// test if "b" is the same as this;
	if ( this == &b )
	{
		return( *this );
	}
	
	if ( b.GetSize() > 0 )
	{
		if ( ( err = CopyFromPointer( b.GetPointer(), b.GetSize() ) ) !=SUCCESS_SUCCESS )
		{
			Flush();
			
			ThrowMapiException(err);
		}
	}
	else
	{
		Flush();
	}
	return( *this );
}

/////////////////////////////////////////////////////////////////
// += will concat to CMapiRecip objects
// 
// can throw a CMapiException
//
#pragma warning ( disable : 4135 )
CMapiRecip& CMapiRecip::operator+=( CMapiRecip& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	DWORD err;
	
	if ( this == &b )
	{
		return( *this );
	}
	
	CMapiRecip dest;
	
	if ( b.GetSize() == 0 )
	{
		return( *this );
	}
	
	if ( GetSize() == 0 )
	{
		*this = b;
		return( *this );
	}
	
	if ( ( err = dest.CreatePointer( (WORD)( GetSize() + b.GetSize() ) ) ) != SUCCESS_SUCCESS )  // b may be larger than 1 recipenent
	{
		ThrowMapiException(err);		
		return( *this );
	}
	
	int i;
	for ( i=0; i<(int)GetSize(); ++i )
	{
		if ( ( err = SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		if ( ( err = dest.SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		
		if ( ( err = dest.SetName( GetName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		if ( ( err = dest.SetAddress( GetAddress() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		if ( ( err = dest.SetEntryID( m_pPosition->lpEntryID, (WORD)m_pPosition->ulEIDSize ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		if ( ( err = dest.SetRecipClass( GetRecipClass() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		dest.m_pPosition->ulReserved = m_pPosition->ulReserved;
	}
	
	int j = 0;
	for ( i = GetSize(); i<(int)dest.GetSize(); ++i )
	{
		if ( ( err = dest.SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
		if ( ( err = b.SetPosition( j ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
		
			
		if ( ( err = dest.SetName( b.GetName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}

		if ( ( err = dest.SetAddress( b.GetAddress() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}

		if ( ( err = dest.SetEntryID( b.m_pPosition->lpEntryID, (WORD)b.m_pPosition->ulEIDSize ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}

		if ( ( err = dest.SetRecipClass( b.GetRecipClass() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}

		dest.m_pPosition->ulReserved = b.m_pPosition->ulReserved;
		
		++j;
	}
	
	TRY
	{
		*this = dest;  // copy the new recip class into this.
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	return( *this );
}


////////////////////////////////////////////////////////
// Stores and retrieves the object to disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
// 
void CMapiRecip::Serialize( CArchive& ar )
{
 	TRY
 	{
 		CObject::Serialize( ar );
    
	    if( ar.IsStoring() )
    	{
    		ar << *this;
	    }
    	else
	    {
    		ar >> *this;
    	}
    }
    CATCH( CException, e )
    {
    	THROW_LAST();
    }
    END_CATCH
}


//////////////////////////////////////////////////////////
// save the CMapiRecip object to disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
CArchive& AFXAPI operator <<( CArchive& ar, CMapiRecip& recip )
{
	int i;
	CString String;
	
	TRACE( " Serializing to disk a CMapiRecip object\n" );
	
	TRY
	{
		ar << recip.GetSize();
	
		if ( recip.GetSize() > 0 )
		{
			for ( i=0; i < (int)recip.GetSize(); ++i )
			{
				recip.SetPosition( i );
				
				ar << recip.GetRecipClass();
				
				String = recip.GetName();
				ar << String;
				
				String = recip.GetAddress();
				ar << String;
			}
		}
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	return( ar );
}

////////////////////////////////////////////////////////////////
// retrieve the CMapiRecip object from disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
CArchive& AFXAPI operator >>( CArchive& ar, CMapiRecip& recip )
{
	int i;
	DWORD dwSize, dwClass, err;
	CString string;
	
	TRACE( " Serializing from disk a CMapiRecip object\n" );
		
	recip.Flush();
	TRY
	{
		ar >> dwSize;
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	if ( dwSize > 0 )
	{
		if ( ( err = recip.CreatePointer( (WORD)dwSize ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			recip.Flush();
			return( ar );
		}	
		
		for ( i=0; i < (int)recip.GetSize(); ++i )
		{
			if ( ( err = recip.SetPosition( i ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				recip.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> dwClass;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = recip.SetRecipClass( dwClass ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				recip.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> string;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = recip.SetName( string ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				recip.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> string;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = recip.SetAddress( string ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				recip.Flush();
				return( ar );
			}
		}
	}
	return( ar );
}

#ifdef _DEBUG
//////////////////////////////////////////////////////////
// Dumps the object to AUX.
//
void CMapiRecip::Dump( CDumpContext& dc )
{
	CObject::Dump( dc );
	
	dc << " \nDumping a CMapiRecip object \n\n";
	if ( GetSize() == 0 )
	{
		dc << " CMapiRecip is empty \n";
		return;
	}
	
	int i;
	
	for ( i=0; i<(int)GetSize(); ++i )
	{
		SetPosition( i );
		
		dc << "Element number:- " << i << "\n";
		dc << " RecipClass is " << GetRecipClass() << "\n";
		dc << " Name is " << GetName() << "\n";
		dc << " Address is " << GetAddress() << "\n";
		dc << " Number of bytes in lpEntryID is " << m_pPosition->ulEIDSize << "\n";
		dc << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
	}
	dc << "Finished dumping CMapiRecip object\n\n";
}
#endif  //_DEBUG

	
IMPLEMENT_SERIAL( CMapiFile, CObject, 0 );

////////////////////////////////////////////////////////////////////////
// Creates class and sets members to NULL
//
CMapiFile::CMapiFile()
{
 	ASSERT_VALID( this );
	m_pMapiFileDesc = NULL;
	m_pPosition = NULL;
	m_ulFileCount = 0;
}

//////////////////////////////////////////////////////////////////////
// Copy constructor Can throw a CMapiException
//
CMapiFile::CMapiFile( CMapiFile& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	
	m_pMapiFileDesc = NULL;
	m_pPosition = NULL;
	m_ulFileCount = 0;

	if ( b.GetSize() == 0 )
	{
		return;
	}
	
	TRY
	{
		*this = b;
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
}


//////////////////////////////////////////////////////////////
//	CMapiFile::~CMapiFile
//
//	Deletes all memory assoicated with the object.
//
CMapiFile::~CMapiFile()
{
	ASSERT_VALID( this );
	Flush();
}

///////////////////////////////////////////////////////////////////////
//	CMapiFile::Flush()
//  
//  frees all memory associated with this object.
//
void CMapiFile::Flush()
{
	ASSERT_VALID( this );
	if ( m_pMapiFileDesc != NULL )
	{
		m_pPosition = m_pMapiFileDesc;
		
		ULONG i;
		for ( i=0 ; i<m_ulFileCount ; ++i )
		{
			delete m_pPosition->lpszPathName;
			m_pPosition->lpszPathName = NULL;

			delete m_pPosition->lpszFileName;
			m_pPosition->lpszFileName = NULL;

			++m_pPosition;
		}
		delete[] m_pMapiFileDesc;
		m_pMapiFileDesc = m_pPosition = NULL;
	}
	m_ulFileCount = 0;
}

/////////////////////////////////////////////////////////////////////
//	CMapiFile::CreatePointer
//	allocates memory for array of MapiFileDesc structures of dwSize
//  and sets contents to NULL
//
//	returns SUCCESS_SUCCESS, POINTER_ALREADY_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiFile::CreatePointer( WORD wSize )
{
	ASSERT_VALID( this );
	ASSERT ( wSize > 0 );
	
	if ( m_pMapiFileDesc == NULL )
	{
	m_pMapiFileDesc = new MapiFileDesc[wSize];
		if ( m_pMapiFileDesc == NULL )
		{
			return( MEMORY_ERROR_ON_NEW );
		}
		m_ulFileCount = (ULONG)wSize;
		
		_fmemset( m_pMapiFileDesc, NULL, ( sizeof(MapiFileDesc) * wSize ) );
		m_pPosition = m_pMapiFileDesc;
	}
	else
	{
		return( POINTER_ALREADY_THERE );		
	}
	return( SUCCESS_SUCCESS );
}		

///////////////////////////////////////////////////////////////////////
// CMapiFile::CopyFromPointer( MapiFileDesc* pMapiFile, DWORD dwCount )
//
//	initialises the class with a pointer returned by a mapi function.
//	For use inside a CMapiMessage object or a CMAPI object.
//
//	returns SUCCESS_SUCCESS, POINTER_ALREADY_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiFile::CopyFromPointer( lpMapiFileDesc pMapiFile, DWORD dwCount )
{
	ASSERT_VALID( this );
	DWORD retval, i, r;
	lpMapiFileDesc pTemp = pMapiFile;
	
	Flush();
	
	if ( pMapiFile == NULL || dwCount == 0 )
	{
		return( SUCCESS_SUCCESS );
	}
	
	retval = CreatePointer( (WORD)dwCount );
	if ( retval != SUCCESS_SUCCESS )
	{
		return( retval );		
	}
	
	for ( i=0 ; i<dwCount ; ++i )
	{
		SetPosition( i );
		
		SetFlags( (DWORD)pTemp->flFlags);
		SetAttachmentPosition( (DWORD) pTemp->nPosition );
		
		retval = SetPathName( pTemp->lpszPathName );
		if ( retval != SUCCESS_SUCCESS )
		{
			//	Clean up all creations to ensure free memory when an error occurs.
			if ( i > 0 )
			{
				for ( r=0 ; r<i ; ++r )
				{
					SetPosition( r );
					SetPathName( "" );
					SetFileName( "" );
				}
			}
			return ( retval );	
		}
		retval = SetFileName( pTemp->lpszFileName );
		if ( retval != SUCCESS_SUCCESS )
		{
			//	Clean up all creations to ensure free memory when an error occurs.
			SetPathName( "" );
			if ( i > 0 )
			{
				for ( r=0 ; r<i ; ++r )
				{
					SetPosition( r );
					SetPathName( "" );
					SetFileName( "" );
				}
			}
			return ( retval );	
		}
		m_pPosition->ulReserved = pTemp->ulReserved;
		m_pPosition->flFlags = pTemp->flFlags;
		++pTemp;
	}
	return( SUCCESS_SUCCESS );
}
  	
//////////////////////////////////////////////
// Gets a pointer to the structure
//
// returns a pointer or NULL
//		
inline lpMapiFileDesc CMapiFile::GetPointer()
{	
	ASSERT_VALID( this );
	return ( m_pMapiFileDesc );
}

///////////////////////////////////////////////
// Gets the size of array of structures.
//
DWORD CMapiFile::GetSize()
{
	ASSERT_VALID( this );
	return ( (DWORD)m_ulFileCount );
} 

inline DWORD CMapiFile::GetFlags()
{	
	ASSERT_VALID( this );
	ASSERT( m_pMapiFileDesc != NULL );
	return ( (DWORD)m_pPosition->flFlags );
}

//////////////////////////////////////////////////////
// Sets the class off the recipent.  See MAPI doco for 
// message class types.
//
// returns SUCCESS_SUCCESS or POINTER_NOT_THERE
//		
DWORD CMapiFile::SetFlags( DWORD dwNewFlags )
{
	ASSERT_VALID( this );
	if ( m_pMapiFileDesc == NULL )
	{
		return ( POINTER_NOT_THERE );
	}
	m_pPosition->flFlags = (ULONG)dwNewFlags;
	return ( SUCCESS_SUCCESS );
}

DWORD CMapiFile::SetAttachmentPosition( DWORD dwAttPos	)
{	
	ASSERT_VALID( this );
	if ( m_pMapiFileDesc == NULL )
	{
		return ( POINTER_NOT_THERE );
	}
	m_pPosition->nPosition = (ULONG) dwAttPos;
	return ( SUCCESS_SUCCESS );
}

inline DWORD CMapiFile::GetAttachmentPosition()
{
	ASSERT_VALID( this );
	return ( (DWORD)m_pPosition->nPosition );
}

		
/////////////////////////////////////////////////////////////////////
//	CMapiFileDesc::SetPosition
//	sets the postion for all other member functions	by the use of an
//  internal pointer.
//
//	returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or NOT_VALID_POSITION
//
DWORD CMapiFile::SetPosition( DWORD dwPos )
{	
	ASSERT_VALID( this );
	ASSERT( dwPos <= m_ulFileCount );

	if ( m_ulFileCount == 0 )
	{
		return( POINTER_NOT_THERE );
		
	}

	// re-initialise the pointer m_pPosition
	m_pPosition = m_pMapiFileDesc;
	
	// set new position
	m_pPosition += dwPos;
	
	return( SUCCESS_SUCCESS );
}

////////////////////////////////////////////////////////
// Gets the Pathname from a MapiFileDesc struct as a
// CString object.
//
//  can throw a CMapiException
// 
CString CMapiFile::GetPathName() 
{	
	if ( m_pMapiFileDesc == NULL )
	{
		ThrowMapiException(POINTER_NOT_THERE );
		
		m_stringRetval = "";
		return( m_stringRetval );
	}
	m_stringRetval = m_pPosition->lpszPathName;
	return ( m_stringRetval );
}

	
/////////////////////////////////////////////////////////////
//	CMapiFile::SetPathName
//
//	Sets the name in a MapiFileDesc structure;
//  returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or MEMORY_ERROR_ON_NEW.
//
DWORD CMapiFile::SetPathName( CString NewPathName )
{	ASSERT_VALID( this );
	if ( m_pMapiFileDesc == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// delete old string if there
	delete m_pPosition->lpszPathName;
	m_pPosition->lpszPathName = NULL;
	
	if ( NewPathName.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}
	
	m_pPosition->lpszPathName = (LPSTR) new char[NewPathName.GetLength() + 1]; 
	if ( m_pPosition->lpszPathName == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pPosition->lpszPathName, NewPathName );
	NewPathName.ReleaseBuffer();
	return ( SUCCESS_SUCCESS );
}

////////////////////////////////////////////////////////
// Gets the filename from a MapiFileDesc struct as a
// CString object.
//
// returns NULL or a pointer to a CString object which
// the user must delete
// 
CString CMapiFile::GetFileName() 
{
	if ( m_pMapiFileDesc == NULL )
	{
		ThrowMapiException( POINTER_NOT_THERE );
							
		m_stringRetval = "";
		return( m_stringRetval );
	}
	m_stringRetval = m_pPosition->lpszFileName;
	return ( m_stringRetval );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiFile::SetFileName
//
//	Sets the address in a MapiFileDesc structur.
//
//	returns SUCCESS_SUCCESS, POINTER_NOT_THERE, or MEMORY_ERROR_ON_NEW
//
DWORD CMapiFile::SetFileName( CString NewFileName )
{	ASSERT_VALID( this );
	if ( m_pMapiFileDesc == NULL )
	{
		return( POINTER_NOT_THERE );		
	}

	// delete old string if there
	delete m_pPosition->lpszFileName;
	m_pPosition->lpszFileName = NULL;
	
	if ( NewFileName.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pPosition->lpszFileName = (LPSTR)new char[NewFileName.GetLength() + 1];
	if ( m_pPosition == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pPosition->lpszFileName, NewFileName );
	NewFileName.ReleaseBuffer();
	return ( SUCCESS_SUCCESS );
}			

////////////////////////////////////////////////////////
// Stores and retrives a CMapiFile object from disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
void CMapiFile::Serialize( CArchive& ar )
{
 	TRY
 	{
 		CObject::Serialize( ar );
    
    	if( ar.IsStoring() )
    	{
    		ar << *this;
    	}
    	else
    	{
    		ar >> *this;
    	}
    }
    CATCH( CException, e )
   	{
   		THROW_LAST();
   	}
   	END_CATCH
}

////////////////////////////////////////////////////////////////
// Sama as SetPosition
// May throw a CMapiException.
//
CMapiFile& CMapiFile::operator[]( DWORD d )
{
	DWORD err;
	ASSERT( d <= m_ulFileCount );
	
	if ( ( err = SetPosition( d ) ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);
	}
	
	return ( *this );
}

//////////////////////////////////////////////////////////////////////////////////////
// equals operator same as copy constructor
//
// May throw a CMapiException
//
CMapiFile& CMapiFile::operator=( CMapiFile& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	DWORD err;
	
	// test for "b" == to this
	if ( this == &b )
	{
		return ( *this );
	}
	
	
	Flush();
	
	if ( b.GetSize() > 0 )
	{
		if ( ( err = CopyFromPointer( b.GetPointer(), b.GetSize() ) ) !=SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			Flush();
		}
	}
	else
	{
		Flush();
	}
	return( *this );
}

////////////////////////////////////////////////////////////////////////////////////////
//  concats two CMapiFile objects
//  may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
#pragma warning ( disable : 4135 )
CMapiFile& CMapiFile::operator+=( CMapiFile& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	
	CMapiFile dest;
	DWORD err;
	
	if ( b.GetSize() == 0 )
	{
		return( *this );
	}
	
	if ( GetSize() == 0 )
	{
		*this = b;
		return( *this );
	}
	
	if ( ( err = dest.CreatePointer( (WORD)(GetSize() + b.GetSize()) ) ) != SUCCESS_SUCCESS ) // b may be larger than 1 recipenent
	{
		ThrowMapiException(err);
		
		return( *this );
	}
		
	
	int i;
	for ( i=0; i<(int)GetSize(); ++i )
	{
		if ( ( err = SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = dest.SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		
		if ( ( err = dest.SetAttachmentPosition( GetAttachmentPosition() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = dest.SetFileName( GetFileName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = dest.SetFlags( GetFlags() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = dest.SetPathName( GetPathName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		
		dest.m_pPosition->ulReserved = m_pPosition->ulReserved;
		dest.m_pPosition->ulReserved = m_pPosition->ulReserved;
	}
	
	int j = 0;
	for ( i = GetSize(); i<(int)dest.GetSize(); ++i )
	{
		if ( ( err = dest.SetPosition( i ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = b.SetPosition( j ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
			
		if ( ( err = dest.SetAttachmentPosition( b.GetAttachmentPosition() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( *this );
		}
			
		if ( ( err = dest.SetFileName( b.GetFileName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
			
		if ( ( err = dest.SetFlags( b.GetFlags() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			
			return( *this );
		}
			
		if ( ( err = dest.SetPathName( b.GetPathName() ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);
			return( *this );
		}
			
		
		dest.m_pPosition->ulReserved = b.m_pPosition->ulReserved;
		dest.m_pPosition->ulReserved = b.m_pPosition->ulReserved;
		
		++j;
	}
	
	
	TRY
	{
		*this = dest;  // copy the new recip class into this.
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	return( *this );
}
	
#ifdef _DEBUG
//////////////////////////////////////////////////////////
//
void CMapiFile::Dump( CDumpContext& dc )
{
	CObject::Dump( dc );
	
	dc << " Dumping a CMapiFile object \n";
	dc << "#############################\n";
	if ( GetSize() == 0 )
	{
		dc << " CMapiFile is empty \n";
		return;
	}
	
	int i;
	
	for ( i=0; i<(int)GetSize(); ++i )
	{
		SetPosition( i );
		
		dc << "Element number:- " << i << "\n";
		dc << " FileName is " << GetFileName() << "\n";
		dc << " PathName is " << GetPathName() << "\n";
		dc << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n";
	}
}
#endif  //_DEBUG

//////////////////////////////////////////////////////////
//  may throw a CMemoryException, CArchiveException, CFileException, CMapiException
CArchive& AFXAPI operator <<( CArchive& ar, CMapiFile& file )
{
	int i;
	CString String;
	
	TRY
	{
		ar << file.GetSize();
	
		if ( file.GetSize() > 0 )
		{
			for ( i=0; i < (int)file.GetSize(); ++i )
			{
				file.SetPosition( i );
				
				ar << file.GetFlags();
				
				ar << file.GetAttachmentPosition();
				
				String = file.GetPathName();
				ar << String;
				
				String = file.GetFileName();
				ar << String;
			}
		}
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	 
	return( ar );
}

//////////////////////////////////////////////////////////////////////////
// Saves a CMapiFile object to disk
// 
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
CArchive& AFXAPI operator >>( CArchive& ar, CMapiFile& file )
{
	int i;
	DWORD dwFlags, dwPOS, dwSize, err;
	CString string;
	
	file.Flush();
	
	TRY
	{
		ar >> dwSize;
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	if ( dwSize > 0 )
	{
		if ( ( err = file.CreatePointer( (WORD)dwSize ) ) != SUCCESS_SUCCESS )
		{
			ThrowMapiException(err);			
			return( ar );
		}
		
		for ( i=0; i < (int)file.GetSize(); ++i )
		{
			if ( ( err = file.SetPosition( i ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				file.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> dwFlags;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = file.SetFlags( dwFlags ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				file.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> dwPOS;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = file.SetAttachmentPosition( dwPOS ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				file.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> string;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = file.SetPathName( string ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				file.Flush();
				return( ar );
			}
			
			TRY
			{
				ar >> string;
			}
			CATCH( CException, e )
			{
				THROW_LAST();
			}
			END_CATCH
			if ( ( err = file.SetFileName( string ) ) != SUCCESS_SUCCESS )
			{
				ThrowMapiException(err);
				file.Flush();
				return( ar );
			}
		}
	}
	return( ar );
}

///////////////////////////////////////////////////
//
IMPLEMENT_SERIAL( CMapiMessage, CObject, 2 );

/////////////////////////////////////////////////////////////////////
//CMapiMessage::CMapiMessage()
//
CMapiMessage::CMapiMessage()
{
	ASSERT_VALID( this );
	DWORD err;
	
	m_pMapiMessage = NULL;
	if ( ( err = CreatePointer() ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);
	}
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor for CMapiMessage
// 
// can throw a CMapiException
//
CMapiMessage::CMapiMessage( CMapiMessage& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );
	
	DWORD err;
	
	m_pMapiMessage = NULL;
	
	if ( ( err = CopyFromPointer( b.GetPointer() ) ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);
	}
}


///////////////////////////////////////////////////////////////////////////////
//CMapiMessage::~CMapiMessage()
//
CMapiMessage::~CMapiMessage()
{
	ASSERT_VALID( this );
	Flush();
}

/////////////////////////////////////////////////////////////////////////////////////
// cleans out all memory allocated by CMapiMessage. called in distructor
//
void CMapiMessage::Flush()
{
	ASSERT_VALID( this );
	if ( m_pMapiMessage != NULL )
	{
		delete m_pMapiMessage->lpszSubject;
		m_pMapiMessage->lpszSubject = NULL;
		
		delete m_pMapiMessage->lpszNoteText;
		m_pMapiMessage->lpszNoteText = NULL;
		
		delete m_pMapiMessage->lpszMessageType;
		m_pMapiMessage->lpszMessageType = NULL;
		
		delete m_pMapiMessage->lpszDateReceived;
		m_pMapiMessage->lpszDateReceived = NULL;
		
		delete m_pMapiMessage->lpszConversationID;
		m_pMapiMessage->lpszConversationID = NULL;

	delete[] m_pMapiMessage;
	m_pMapiMessage = NULL;
	m_Orig.Flush();
	m_Recip.Flush();
	m_File.Flush();
	}
}

///////////////////////////////////////////////////////////////////////////
//	CMapiMessage::CreatePointer();
//
//
DWORD CMapiMessage::CreatePointer()
{
	ASSERT_VALID( this );
	
	if ( m_pMapiMessage == NULL )
	{
	m_pMapiMessage = new MapiMessage;
		if ( m_pMapiMessage == NULL )
		{
			return( MEMORY_ERROR_ON_NEW );
		}
		
		_fmemset( m_pMapiMessage, NULL, sizeof( MapiMessage ) );
	}
	else
	{
		return( POINTER_ALREADY_THERE );		
	}
	return( SUCCESS_SUCCESS );
}		

//////////////////////////////////////////////////////////////////////////
//CMapiMessage::CopyFromPointer( MapiMessage* pMessage,
//
// Takes a pointer to a MapiMessage struct
//
DWORD CMapiMessage::CopyFromPointer( lpMapiMessage pMessage )
{
	ASSERT_VALID( this );
	DWORD retval;
	
	if ( pMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	if ( m_pMapiMessage != NULL )
	{
		return( POINTER_ALREADY_THERE );		
	}
	
	if ( ( retval = CreatePointer() ) != SUCCESS_SUCCESS )
	{
		return( retval );
	}
			
	if ( ( retval = SetSubject( pMessage->lpszSubject ) ) != SUCCESS_SUCCESS )
	{
		delete m_pMapiMessage;
		m_pMapiMessage = NULL;
		return( retval );		
	}
	
	if ( ( retval = SetNoteText( pMessage->lpszNoteText ) ) != SUCCESS_SUCCESS )
	{
		SetSubject( "" );
		delete m_pMapiMessage;
		m_pMapiMessage = NULL;
		return( retval );		
	}

	if ( ( retval = SetMessageType( pMessage->lpszMessageType ) ) != SUCCESS_SUCCESS )
	{
		SetSubject( "" );
		SetNoteText( "" );
		delete m_pMapiMessage;
		m_pMapiMessage = NULL;
		return( retval );		
	}
	
	if ( pMessage->lpszDateReceived != NULL ) // test if the is something there to copy.
	{
		m_pMapiMessage->lpszDateReceived = new char[ _fstrlen( pMessage->lpszDateReceived ) + 1 ];
		if ( m_pMapiMessage->lpszDateReceived == NULL )
		{
			SetSubject( "" );
			SetNoteText( "" );
			SetMessageType( "" );
			delete m_pMapiMessage;
			m_pMapiMessage = NULL;
			return( MEMORY_ERROR_ON_NEW );		
		}
		_fstrcpy( m_pMapiMessage->lpszDateReceived, pMessage->lpszDateReceived );
	}
	
	if ( ( retval = SetConversationID( pMessage->lpszConversationID ) ) != SUCCESS_SUCCESS )
	{
		SetSubject( "" );
		SetNoteText( "" );
		delete m_pMapiMessage->lpszDateReceived;
		delete m_pMapiMessage;
		m_pMapiMessage = NULL;
		return( retval );		
	}
	
	m_pMapiMessage->flFlags = pMessage->flFlags;
	
	if ( ( retval = m_Orig.CopyFromPointer( pMessage->lpOriginator, 1 ) ) != SUCCESS_SUCCESS )
	{
		SetSubject( "" );
		SetNoteText( "" );
		delete m_pMapiMessage->lpszDateReceived;
		SetConversationID( "" );
		delete m_pMapiMessage;
		m_pMapiMessage = NULL;
		return( retval );		
	}
	m_pMapiMessage->lpOriginator = m_Orig.GetPointer();
	
	if ( pMessage->nRecipCount > 0 )
	{
		retval = m_Recip.CopyFromPointer( pMessage->lpRecips, pMessage->nRecipCount );
		if ( retval != SUCCESS_SUCCESS )
		{
			SetSubject( "" );
			SetNoteText( "" );
			delete m_pMapiMessage->lpszDateReceived;
			SetConversationID( "" );
			m_Orig.Flush();
			delete m_pMapiMessage;
			m_pMapiMessage = NULL;
			return( retval );		
		}
	}
	
	m_pMapiMessage->nRecipCount = m_Recip.GetSize();
	m_pMapiMessage->lpRecips = m_Recip.GetPointer();

	if ( pMessage->nFileCount > 0 )
	{
		retval = m_File.CopyFromPointer( pMessage->lpFiles, pMessage->nFileCount );
		if ( retval != SUCCESS_SUCCESS )
		{
			SetSubject( "" );
			SetNoteText( "" );
			delete m_pMapiMessage->lpszDateReceived;
			SetConversationID( "" );
			m_Orig.Flush();
			m_Recip.Flush();
			delete m_pMapiMessage;
			m_pMapiMessage = NULL;
			return( retval );		
		}
	}
	
	m_pMapiMessage->nFileCount = m_File.GetSize();
	m_pMapiMessage->lpFiles = m_File.GetPointer();
	
	m_pMapiMessage->ulReserved = pMessage->ulReserved;
	
	return( SUCCESS_SUCCESS );
}
	
inline lpMapiMessage CMapiMessage::GetPointer()
{	
	ASSERT_VALID( this );
	return( m_pMapiMessage );
}

////////////////////////////////////////////////////////
// Gets the subject from a MapiMessage struct as a
// CString object.
//
// Can throw a CMapiException 
// 
CString CMapiMessage::GetSubject() 
{	
	if ( m_pMapiMessage == NULL )
	{
			ThrowMapiException( POINTER_NOT_THERE );
			
			m_stringRetval = "";
			return( m_stringRetval );
	}
	m_stringRetval = m_pMapiMessage->lpszSubject;
	return( m_stringRetval );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiMessage::SetSubject
//
//	Sets the subject in a MapiMessage structur.
//
//	returns SUCCESS_SUCCESS if successfull else error code.
//
DWORD CMapiMessage::SetSubject( CString Subject )
{	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// Delete old string if there;
	delete m_pMapiMessage->lpszSubject;
	m_pMapiMessage->lpszSubject = NULL;
		
	if ( Subject.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pMapiMessage->lpszSubject = (LPSTR)new char[Subject.GetLength() + 1];
	if ( m_pMapiMessage->lpszSubject == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pMapiMessage->lpszSubject, Subject );
	return ( SUCCESS_SUCCESS );
}			

////////////////////////////////////////////////////////
// Gets the NoteText from a MapiMessage struct as a
// CString object.
//
// can throw a CMapiException
//
CString CMapiMessage::GetNoteText() 
{
	if ( m_pMapiMessage == NULL )
	{
			ThrowMapiException( POINTER_NOT_THERE );
								
			m_stringRetval = "";
			return( m_stringRetval );
	}
	m_stringRetval = m_pMapiMessage->lpszNoteText;
	return( m_stringRetval );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiMessage::SetNoteText
//
//	Sets the subject in a MapiMessage structur.
//
//	returns SUCCESS_SUCCESS if successfull else error code.
//
DWORD CMapiMessage::SetNoteText( CString NoteText )
{	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// Delete old string if there;
	delete m_pMapiMessage->lpszNoteText;
	m_pMapiMessage->lpszNoteText = NULL;
	
	if ( NoteText.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pMapiMessage->lpszNoteText = (LPSTR)new char[NoteText.GetLength() + 1];
	if ( m_pMapiMessage->lpszNoteText == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pMapiMessage->lpszNoteText, NoteText );
	return ( SUCCESS_SUCCESS );
}			

////////////////////////////////////////////////////////
// Gets the MessageType from a MapiMessage struct as a
// CString object.
//
// Can throw a CMapiException
//
CString CMapiMessage::GetMessageType() 
{	
	if ( m_pMapiMessage == NULL )
	{
		ThrowMapiException( POINTER_NOT_THERE );
							
		m_stringRetval = "";
		return( m_stringRetval );
	}
	m_stringRetval = m_pMapiMessage->lpszMessageType;
	return( m_stringRetval );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiMessage::SetMessageType
//
//	Sets the subject in a MapiMessage structur.
//
//	returns SUCCESS_SUCCESS if successfull else error code.
//
DWORD CMapiMessage::SetMessageType( CString MessageType )
{	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// Delete old string if there;
	delete m_pMapiMessage->lpszMessageType;
	m_pMapiMessage->lpszMessageType = NULL;
	
	if ( MessageType.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pMapiMessage->lpszMessageType = (LPSTR)new char[MessageType.GetLength() + 1];
	if ( m_pMapiMessage->lpszMessageType == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pMapiMessage->lpszMessageType, MessageType );
	return ( SUCCESS_SUCCESS );
}			

//////////////////////////////////////////////////////////////////////
//	DWORD CMapiMessage::GetTimeReceived(CTime& RecieveTime);
//
DWORD CMapiMessage::GetTimeReceived( CTime& RecevTime)
{
	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	if ( m_pMapiMessage->lpszDateReceived == NULL )
	{
		return( POINTER_NOT_THERE );
	}
	CString tempstring = m_pMapiMessage->lpszDateReceived;
	
	CTime retval( atoi(tempstring.Mid( 0, 4 )),			// extract year
				  atoi(tempstring.Mid( 5, 2 )),			// extract month
				  atoi(tempstring.Mid( 8, 2 )),			// extract day
				  atoi(tempstring.Mid( 11, 2 )),		// extract hour
				  atoi(tempstring.Mid( 14, 2 )), 0);	// extract minuet
	
	RecevTime = retval;
	
	return( SUCCESS_SUCCESS );
}

/////////////////////////////////////////////////////////////////////////
// DWORD CMapiMessage::SetTimeReceived( CTime& time )
//
//	This function exists to support archiving of messages.
DWORD CMapiMessage::SetTimeReceived( CTime time )
{
	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );
	}
	else
	{
		delete m_pMapiMessage->lpszDateReceived;
		
		m_pMapiMessage->lpszDateReceived = new char[17];
		if ( m_pMapiMessage->lpszDateReceived == NULL )
		{
			return( MEMORY_ERROR_ON_NEW );
		}
		
		CString strTime;
		
		wsprintf(strTime.GetBuffer(20), "%i/%02i/%02i %02i:%02i", 
			time.GetYear(), time.GetMonth(), time.GetDay(), time.GetHour(), time.GetMinute());
		strTime.ReleaseBuffer();				
		
		_fstrcpy( m_pMapiMessage->lpszDateReceived, strTime);
		
		return ( SUCCESS_SUCCESS );
	}
}

////////////////////////////////////////////////////////
// Gets the ConversationID from a MapiMessage struct as a
// CString object.
//
// can throw a CMapiException
//
CString CMapiMessage::GetConversationID() 
{	
	if ( m_pMapiMessage == NULL )
	{
			ThrowMapiException( POINTER_NOT_THERE );
								
			m_stringRetval = "";
			return( m_stringRetval );
	}
	m_stringRetval = m_pMapiMessage->lpszConversationID;
	return( m_stringRetval );
}

////////////////////////////////////////////////////////////////////////////
//	CMapiMessage::SetConversationID
//
//	Sets the subject in a MapiMessage structur.
//
//	returns SUCCESS_SUCCESS if successfull else error code.
//
DWORD CMapiMessage::SetConversationID( CString ConversationID )
{	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	// Delete old string if there;
	delete m_pMapiMessage->lpszConversationID;
	m_pMapiMessage->lpszConversationID = NULL;
	
	if ( ConversationID.GetLength() == 0 )
	{
		return ( SUCCESS_SUCCESS );
	}

	m_pMapiMessage->lpszConversationID = (LPSTR)new char[ConversationID.GetLength() + 1];
	if ( m_pMapiMessage->lpszConversationID == NULL )
	{
		return ( MEMORY_ERROR_ON_NEW );
	}
	_fstrcpy( m_pMapiMessage->lpszConversationID, ConversationID );
	return ( SUCCESS_SUCCESS );
}			

/////////////////////////////////////////////////////////////////////////
//	CMapiRecip& CMapiMessage::GetOriginator( CMapiRecip* Orig );
//
CMapiRecip& CMapiMessage::GetOriginator()
{
	ASSERT_VALID( this );
	
	return( m_Orig );
}

/////////////////////////////////////////////////////////////////////////////
//  For archiving only.
//
DWORD CMapiMessage::SetOriginator( CMapiRecip& recip )
{
	ASSERT_VALID( this );
	
	if ( m_pMapiMessage == NULL )
	{
		return ( POINTER_NOT_THERE );
	}
	TRY
	{
		m_Orig = recip;
	}
	CATCH( CMapiException, e )
	{
		m_pMapiMessage->lpOriginator = NULL;
		return( e->m_dwErrorCode );
	}
	END_CATCH
	
	m_pMapiMessage->lpOriginator = m_Orig.GetPointer();
	return( SUCCESS_SUCCESS );
}
	
/////////////////////////////////////////////////////////////////////////
//	CMapiRecip& CMapiMessage::GetRecips( CMapiRecip* pRecip);
//
CMapiRecip& CMapiMessage::GetRecips()
{
	ASSERT_VALID( this );
	
	return( m_Recip );
}
	
//////////////////////////////////////////////////////
// DWORD CMapiMessage::SetRecips( CMapiRecip& Recip )
//	
DWORD CMapiMessage::SetRecips( CMapiRecip& Recip )
{
	ASSERT_VALID( this );
	DWORD retval;
	
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	retval = m_Recip.CopyFromPointer( Recip.GetPointer(), Recip.GetSize() );
	if ( retval != SUCCESS_SUCCESS )
	{
		return( retval );
	}
	
	m_pMapiMessage->nRecipCount = m_Recip.GetSize();
	m_pMapiMessage->lpRecips = m_Recip.GetPointer();
	
	return( SUCCESS_SUCCESS );
}

/////////////////////////////////////////////////////////////////////////
//	CMapiFile& CMapiMessage::GetFiles( CMapiFile* pFile);
//
CMapiFile& CMapiMessage::GetFiles()
{
	ASSERT_VALID( this );
	
	return( m_File );
}
	
//////////////////////////////////////////////////////
// DWORD CMapiMessage::SetFiles( CMapiFile& File )
//	
DWORD CMapiMessage::SetFiles( CMapiFile& File )
{
	ASSERT_VALID( this );
	DWORD retval;
	
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );		
	}
	
	retval = m_File.CopyFromPointer( File.GetPointer(), File.GetSize() );
	if ( retval != SUCCESS_SUCCESS )
	{
		return( retval );
	}
	
	m_pMapiMessage->nFileCount = m_File.GetSize();
	m_pMapiMessage->lpFiles = m_File.GetPointer();
	
	return( SUCCESS_SUCCESS );
}

///////////////////////////////////////////////////////////////////////////
//  equals operator same as copy constructor 
// can throw a CMapiException
CMapiMessage& CMapiMessage::operator=( CMapiMessage& b )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &b );

	DWORD err;
	
	// Test if "b"==this
	if ( this == &b )
	{
		return ( *this );
	}
	
    Flush();
	if ( ( err = CopyFromPointer( b.GetPointer() ) ) != SUCCESS_SUCCESS )
	{
		ThrowMapiException(err);
	}
	
	return ( *this );
}

//////////////////////////////////////////////////////////////
//  Gets the message flags 
//
//	see Mapi technical referance for flag details
//
DWORD CMapiMessage::GetFlags()
{	
	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );
	}
	else 
	{
		return( (DWORD)m_pMapiMessage->flFlags );
	}
}
		
/////////////////////////////////////////////////////////////////
//  Sets message Flags
//
//  see mapi Technical referance for flag details
//
DWORD CMapiMessage::SetFlags( DWORD dwFlags )
{	
	ASSERT_VALID( this );
	if ( m_pMapiMessage == NULL )
	{
		return( POINTER_NOT_THERE );
	}
	else
	{
		m_pMapiMessage->flFlags = (ULONG)dwFlags;
		return( SUCCESS_SUCCESS );
	}
}

//////////////////////////////////////////////////////////////////////////////////
// Saves a CMapiMessage object to disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
CArchive& AFXAPI operator <<( CArchive& ar, CMapiMessage& message )
{
	
	CString String;
	
    TRY
	{
		String = message.GetSubject();
    	ar << String;
    
    	String = message.GetNoteText();
    	ar << String;
    
    	String = message.GetMessageType();
    	ar << String;
    
    	CTime time;
		message.GetTimeReceived( time );
		ar << time;
	
		String = message.GetConversationID();
		ar << String;
	
		ar << message.GetFlags();
	
		ar << message.GetOriginator();
	
		ar << message.GetRecips();
	
		ar << message.GetFiles();
	}
	CATCH( CException, e )
	{
		THROW_LAST();
	}
	END_CATCH
	
	return ( ar );
}

//////////////////////////////////////////////////////////////////////////////////
// Retrieves a CMapiObject from disk
// may throw a CMemoryException, CArchiveException, CFileException, CMapiException
//
CArchive& AFXAPI operator >>( CArchive& ar, CMapiMessage& message )
{
	CString string;
	CMapiRecip recip;
	CMapiFile file;
	DWORD dwFlags;
	
    TRY
	{
		ar >> string;
    	message.SetSubject( string );
    
    	ar >> string;
    	message.SetNoteText( string );
    
    	ar >> string;
    	message.SetMessageType( string );
    
    	CTime time;
		ar >> time;
		message.SetTimeReceived( time );
	
		ar >> string;
		message.SetConversationID( string );
	
		ar >> dwFlags;
		message.SetFlags( dwFlags );
	
		ar >> recip;
		message.SetOriginator( recip );
	
		recip.Flush();
		ar >> recip;
		message.SetRecips( recip );
	
		ar >> file;
		message.SetFiles( file );
	}
	CATCH( CException, e )
	{
		message.Flush();
		THROW_LAST();
	}
	END_CATCH
		
	return ( ar );
}

////////////////////////////////////////////
//
// Call 'IMPLEMENT_DYNAMIC' macro for all the
//  classes declared in clasmapi.h

IMPLEMENT_DYNAMIC( CMAPI, CObject );

tLogonProc			CMAPI::lpfnMAPILogon;
tLogoffProc			CMAPI::lpfnMAPILogoff;
tSendMailProc		CMAPI::lpfnMAPISendMail;
tSendDocumentsProc	CMAPI::lpfnMAPISendDocuments;
tFindNextProc		CMAPI::lpfnMAPIFindNext;
tReadMailProc		CMAPI::lpfnMAPIReadMail;
tSaveMailProc		CMAPI::lpfnMAPISaveMail;
tDeleteMailProc		CMAPI::lpfnMAPIDeleteMail;
tAddressProc		CMAPI::lpfnMAPIAddress;
tResolveNameProc	CMAPI::lpfnMAPIResolveName;
tDetailsProc		CMAPI::lpfnMAPIDetails;
tFreeBufferProc		CMAPI::lpfnMAPIFreeBuffer;
HINSTANCE			CMAPI::hinstMAPIdll;
UINT 				CMAPI::m_CMAPI_InstanceCount = 0;


CMAPI::CMAPI()
{
	ASSERT_VALID( this );
	
	m_pParent = NULL;
	m_lhSession = NULL;
	if ( !m_CMAPI_InstanceCount )
	{
		DWORD err;
		
		if ( ( err = InItMAPI() ) != 0  )
		{
			ThrowMapiException(err);
		}
		
	}
	++m_CMAPI_InstanceCount;
}



#pragma warning ( disable : 4769 )
CMAPI::~CMAPI()
{
	ULONG retval;
	if(m_lhSession)
		retval = (*lpfnMAPILogoff)( m_lhSession, (ULONG)m_pParent->m_hWnd, 0L, 0L );
	--m_CMAPI_InstanceCount;
	if ( m_CMAPI_InstanceCount == 0 )
	{
		DeInItMAPI();
	}
}

UINT CMAPI::InItMAPI()
{

#ifndef _WIN32
	if ((hinstMAPIdll = ::LoadLibrary("MAPI.DLL")) < (HINSTANCE)32)
    	return(ERR_LOAD_LIB);
	if ( ( lpfnMAPILogon = (tLogonProc)GetProcAddress(hinstMAPIdll,"MAPILogon") ) == NULL )
		return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPILogoff = (tLogoffProc)GetProcAddress(hinstMAPIdll,"MAPILogOff") ) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPISendMail = (tSendMailProc)GetProcAddress(hinstMAPIdll,"MAPISendMail")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPISendDocuments = (tSendDocumentsProc)GetProcAddress(hinstMAPIdll,"MAPISendDocuments")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPIFindNext = (tFindNextProc)GetProcAddress(hinstMAPIdll,"MAPIFindNext")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPIReadMail = (tReadMailProc)GetProcAddress(hinstMAPIdll,"MAPIReadMail")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPISaveMail = (tSaveMailProc)GetProcAddress(hinstMAPIdll,"MAPISaveMail")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPIDeleteMail = (tDeleteMailProc)GetProcAddress(hinstMAPIdll,"MAPIDeleteMail")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPIFreeBuffer = (tFreeBufferProc)GetProcAddress(hinstMAPIdll,"MAPIFreeBuffer")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ((lpfnMAPIAddress = (tAddressProc)GetProcAddress(hinstMAPIdll,"MAPIAddress")) == NULL)
    	return(ERR_LOAD_FUNC);

	if ( ( lpfnMAPIDetails = (tDetailsProc)GetProcAddress(hinstMAPIdll,"MAPIDetails")) == NULL)
    	return(ERR_LOAD_FUNC);
		
	if ( ( lpfnMAPIResolveName= (tResolveNameProc)GetProcAddress(hinstMAPIdll,"MAPIResolveName")) == NULL)
    	return(ERR_LOAD_FUNC);
#else
	if ((hinstMAPIdll = ::LoadLibrary("MAPI32.DLL")) == NULL )	
		return(GetLastError());
	if ( ( lpfnMAPILogon = (tLogonProc)GetProcAddress(hinstMAPIdll,"MAPILogon") ) == NULL )
		return(GetLastError());

	if ( ( lpfnMAPILogoff = (tLogoffProc)GetProcAddress(hinstMAPIdll,"MAPILogoff") ) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPISendMail = (tSendMailProc)GetProcAddress(hinstMAPIdll,"MAPISendMail")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPISendDocuments = (tSendDocumentsProc)GetProcAddress(hinstMAPIdll,"MAPISendDocuments")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPIFindNext = (tFindNextProc)GetProcAddress(hinstMAPIdll,"MAPIFindNext")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPIReadMail = (tReadMailProc)GetProcAddress(hinstMAPIdll,"MAPIReadMail")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPISaveMail = (tSaveMailProc)GetProcAddress(hinstMAPIdll,"MAPISaveMail")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPIDeleteMail = (tDeleteMailProc)GetProcAddress(hinstMAPIdll,"MAPIDeleteMail")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPIFreeBuffer = (tFreeBufferProc)GetProcAddress(hinstMAPIdll,"MAPIFreeBuffer")) == NULL)
    	return(GetLastError());

	if ((lpfnMAPIAddress = (tAddressProc)GetProcAddress(hinstMAPIdll,"MAPIAddress")) == NULL)
    	return(GetLastError());

	if ( ( lpfnMAPIDetails = (tDetailsProc)GetProcAddress(hinstMAPIdll,"MAPIDetails")) == NULL)
    	return(GetLastError());
		
	if ( ( lpfnMAPIResolveName= (tResolveNameProc)GetProcAddress(hinstMAPIdll,"MAPIResolveName")) == NULL)
    	return(GetLastError());
#endif //_WIN32	
		

  return(0);
}


void CMAPI::DeInItMAPI()
{
  lpfnMAPILogon = NULL;
  lpfnMAPILogoff= NULL;
  lpfnMAPISendMail= NULL;
  lpfnMAPISendDocuments= NULL;
  lpfnMAPIFindNext= NULL;
  lpfnMAPIReadMail= NULL;
  lpfnMAPISaveMail= NULL;
  lpfnMAPIDeleteMail= NULL;
  lpfnMAPIFreeBuffer = NULL;
  lpfnMAPIAddress= NULL;
  lpfnMAPIDetails = NULL;
  lpfnMAPIResolveName = NULL;

  ::FreeLibrary(hinstMAPIdll);
}	

//////////////////////////////////////////////////////////////////////////////////
// Logs on to a MAIL session, see MAPI technical referance for flag details
// may throw a CMapiException exception
//#pragma warning ( disable : 4769 )
DWORD CMAPI::Logon( LPCSTR lpcstrName, LPCSTR lpcstrPassword, DWORD dwFlags, HWND hWndParent )
{
	DWORD err = SUCCESS_SUCCESS;
    m_pParent = CWnd::GetActiveWindow(); //pParent;
    
	err = (*lpfnMAPILogon)( (ULONG)hWndParent, (LPSTR)lpcstrName, 
		(LPSTR)lpcstrPassword, dwFlags, 0L, &m_lhSession );
	
	if ( err != SUCCESS_SUCCESS )
	{
		m_lhSession = NULL;		
		ThrowMapiException(err);
	}
	return( (DWORD)err );
}
////////////////////////////////////////////////////////////////////////
//  used to free memory allocated by any mapi call
//
inline DWORD CMAPI::FreeBuffer( LPVOID p )
{	
	ASSERT_VALID( this );
	if ( p == NULL )
		return SUCCESS_SUCCESS;
#ifdef _DEBUG
	DWORD retval;
	retval = (DWORD)(*lpfnMAPIFreeBuffer)( (LPVOID) p );
	return( retval );
#else
	return ( (DWORD)(*lpfnMAPIFreeBuffer)( (LPVOID) p ) );
#endif
}



/////////////////////////////////////////////////////////////////
//	CMAPI::Address	
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	Any info in the CMapiRecip object NewRecip will be lost, the
//	Recip object will remain unchanged.
//	
//	Returns can be any MAPI error or any of the errors returned
//	by the CMapiRecip, CMapiFile, or CMapiMessage member functions
//
DWORD CMAPI::Address( CString& Caption, DWORD dwEditFields, 
					  CString& Labels, CMapiRecip& Recip,
					  DWORD dwFlags, CMapiRecip& NewRecip, CWnd *pParent /*= NULL*/ )
{
	ASSERT_VALID( this );
	
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}
	
	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}
	ULONG retval;
	lpMapiRecipDesc pNewRecips;
	ULONG		    ulNewRecipCount;
	
	//Make srings null terminated.
	
	NewRecip.Flush();
	
	retval = (*lpfnMAPIAddress)( m_lhSession, (ULONG)pParent->m_hWnd,
								 Caption.GetBuffer(Caption.GetLength()), (ULONG)dwEditFields,
								 Labels.GetBuffer(Labels.GetLength()), (ULONG)Recip.GetSize(),
								 Recip.GetPointer(), (FLAGS)dwFlags, 0L,
								 &ulNewRecipCount, &pNewRecips );
	Caption.ReleaseBuffer();
	Labels.ReleaseBuffer();
	
	if ( retval == SUCCESS_SUCCESS & pNewRecips == NULL )
	{
		return( (DWORD)retval );
	}

	if ( retval != SUCCESS_SUCCESS )
	{
		FreeBuffer ( pNewRecips );
		ThrowMapiException(retval);
		return( (DWORD)retval );
	}
	if ( ulNewRecipCount == 0 )
	{
		FreeBuffer ( pNewRecips );
		return( (DWORD)retval );
	}
	
	retval = NewRecip.CopyFromPointer( pNewRecips, ulNewRecipCount );
	if ( retval != SUCCESS_SUCCESS )
	{
		FreeBuffer ( pNewRecips );
		return( (DWORD)retval );
	}
	
	FreeBuffer ( pNewRecips );
	return( (DWORD)retval );	
}

//////////////////////////////////////////////////////////////////
//	CMAPI::DeleteMail
//
//	MessageID is returned by CMAPI::FindNext or CMAPI::SaveMail
//
//	Returns can be any MAPI error.
//
DWORD CMAPI::DeleteMail( CString& MessageID, CWnd *pParent /*= NULL*/ )
{	
	ASSERT_VALID( this );
	
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}

	ULONG retval;
	
	retval = (*lpfnMAPIDeleteMail)( m_lhSession, (ULONG)pParent->m_hWnd,
								    MessageID.GetBuffer(MessageID.GetLength()),
								    0L, 0L );
	MessageID.ReleaseBuffer();
	return ( (DWORD)retval );
}

///////////////////////////////////////////////////////////////////////
//	CMAPI::Details
//
//	Recip contains one users details
//
//	Returns can be any MAPI error.
//
DWORD CMAPI::Details( CMapiRecip& Recip, DWORD dwFlags, CWnd *pParent /*= NULL*/ )
{	ASSERT_VALID( this );

	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}
	
	ULONG retval;
	retval = (*lpfnMAPIDetails)( m_lhSession, (ULONG)pParent->m_hWnd,
								 Recip.GetPointer(), (FLAGS)dwFlags, 0L );
	return ( (DWORD)retval );
}
	
/////////////////////////////////////////////////////////////////
//	CMAPI::FindNext
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	
//	Returns can be any MAPI error or any of the errors returned
//	by the CMapiRecip, CMapiFile, or CMapiMessage member functions
//
DWORD CMAPI::FindNext( CString& MessageID, CString& MessageType,
					   DWORD dwFlags, CString& SeedMessageID, CWnd *pParent /*= NULL*/ )
{
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}

	ULONG retval;
	
	retval = (*lpfnMAPIFindNext)( m_lhSession, (ULONG)pParent->m_hWnd,
								  MessageType.GetBuffer(MessageType.GetLength()),
								  SeedMessageID.GetBuffer(SeedMessageID.GetLength()),
								  (FLAGS)dwFlags, 0L,
								  MessageID.GetBuffer(64) );
	MessageType.ReleaseBuffer();
	SeedMessageID.ReleaseBuffer();
	MessageID.ReleaseBuffer();
	return( (DWORD)retval );
}

//////////////////////////////////////////////////////////////////////
//	CMAPI::ReadMail
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	
//	Returns can be any MAPI error or any of the errors returned
//	by the CMapiRecip, CMapiFile, or CMapiMessage member functions
//
DWORD CMAPI::ReadMail ( CString& MessageID, DWORD dwFlags, CMapiMessage& MessageDetails, CWnd *pParent /*= NULL*/ )
{
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}

	ULONG retval;
	lpMapiMessage lpMessage;
	MessageDetails.Flush();
	
	retval = (*lpfnMAPIReadMail)( m_lhSession, (ULONG)pParent->m_hWnd,
								  MessageID.GetBuffer(MessageID.GetLength() ),
								  (FLAGS)dwFlags, 0L, &lpMessage );
	
	MessageID.ReleaseBuffer();
	if ( retval != SUCCESS_SUCCESS )
	{
		FreeBuffer( lpMessage ); 
		return( (DWORD)retval );
	}

	retval = MessageDetails.CopyFromPointer( lpMessage );

	FreeBuffer( lpMessage );
	return( (DWORD)retval );
}


/////////////////////////////////////////////////////////////////////////
//	CMAPI::ResolveName
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	Any info in the CMapiRecip object RecipInfo will be lost.
//	
//	Returns can be any MAPI error or any of the errors returned
//	by the CMapiRecip, CMapiFile, or CMapiMessage member functions
//
DWORD CMAPI::ResolveName( CString& Name, DWORD dwFlags, CMapiRecip& RecipInfo, CWnd *pParent /*= NULL*/ )
{
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}

	if ( Name.GetLength() == 0 )
	{
		return( MAPI_E_NO_RECIP );
	}

	ULONG retval;
	lpMapiRecipDesc lpRecip;
	
	RecipInfo.Flush();
	
	retval = (*lpfnMAPIResolveName)( m_lhSession, (ULONG)pParent->m_hWnd,
									 Name.GetBuffer( Name.GetLength() ),
									 (ULONG)dwFlags, 0L, &lpRecip );
	Name.ReleaseBuffer();
	
	if ( retval != SUCCESS_SUCCESS )
	{
		FreeBuffer( lpRecip );
		return( (DWORD)retval );
	}
	
	retval = (ULONG)RecipInfo.CopyFromPointer( lpRecip, 1 );

	FreeBuffer( lpRecip );
	return( (DWORD)retval );
}

/////////////////////////////////////////////////////////////////////////
//	CMAPI::SaveMail
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	
//	Returns can be any MAPI error.
//
DWORD CMAPI::SaveMail( CMapiMessage& MessageDetails, DWORD dwFlags, CString& MessageID, CWnd *pParent /*= NULL*/ )
{
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		return( MAPI_E_INVALID_SESSION );
	}

	ULONG retval;
	
	retval = (*lpfnMAPISaveMail)( m_lhSession, (ULONG)pParent->m_hWnd,
								  MessageDetails.GetPointer(), (FLAGS)dwFlags,
								  0L, MessageID.GetBuffer(MessageID.GetLength()) );
	MessageID.ReleaseBuffer();
	
	return( (DWORD)retval );
}
	
/////////////////////////////////////////////////////////////////////////
//	CMAPI::SendMail
//
//	The Param's are similar to  those in the MAPI doco and
//	proform the same function.
//	
//	Returns can be any MAPI error.
//
DWORD CMAPI::SendMail( CMapiMessage& MessageDetails, DWORD dwFlags, CWnd* pParent /*= NULL*/ )
{
	if ( pParent == NULL )
	{
		pParent = m_pParent;
	}

	if ( m_lhSession == NULL )
	{
		ThrowMapiException(MAPI_E_INVALID_SESSION);
		return( MAPI_E_INVALID_SESSION );
	}

	DWORD err;
	
	err = (*lpfnMAPISendMail)( m_lhSession, (ULONG)pParent->m_hWnd,
								  MessageDetails.GetPointer(),
								  (FLAGS)dwFlags, 0L );
								  
	if(SUCCESS_SUCCESS != err)
	{
		ThrowMapiException(err);
	}
	return( err );
}


