/////////////////////////////////////////////////////////////////////////////
//  The MAPI classes are designed to provide an MFC shell around the MAPI
//  functions released with MS:Mail 3.x
//	
//	If anyone has questions  or improvements to the classes please let 
//	me know at my CompuServe account.
//  
//  Written by Tony Nathan.  1/6/'93	
//  CompuServe account No. 70302,1332
//



#ifndef __CLASMAPI_H
#define __CLASMAPI_H

#ifndef _WIN32

#ifndef _M_I86LM		// test if large model.
#error The CMAPI class library needs to be used with a large model /AL 
#endif

#else

#define _fmemset memset
#define _fstrcpy strcpy
#define _fstrcat strcat
#define _fstrlen strlen

#endif // _WIN32

#ifndef __cplusplus
#error  must be used as a C++ header file
#endif

#include "stdafx.h"

#define MAPI_E_NO_RECIP						27 //not in mapi.h
#define ERROR_STRING_OFFSET		3300
#define NOT_VALID_POSITION 		96
#define POINTER_NOT_THERE 		97
#define POINTER_ALREADY_THERE 	98
#define MEMORY_ERROR_ON_NEW 	99

#define ERR_LOAD_LIB  0x02
#define ERR_LOAD_FUNC 0x04

#include "mapi.h"


/////////////////////////////////////////////////////////
// the class pass mapi error codes and CMAPI error codes
//
class CMapiException : public CException
{
	DECLARE_DYNAMIC( CMapiException )

	public:
	    CMapiException(DWORD dwEc);		
	    ~CMapiException();
		DWORD m_dwErrorCode;
		CString m_strError;
		
};

void AFXAPI ThrowMapiException(DWORD dwError);

class CMapiRecip : public CObject
{
	DECLARE_SERIAL( CMapiRecip )
    
   	friend class CMapiMessage;   // needs access to private functions;
   	friend class CMAPI;
    
	private:   // Data Members
		CString 		m_stringRetval;
		lpMapiRecipDesc m_pMapiRecipDesc;			
		lpMapiRecipDesc m_pPosition;
		ULONG 		    m_ulRecipCount;
	
	public:		// Constuctors
		CMapiRecip();
		
		CMapiRecip( CMapiRecip& b );
		
		CMapiRecip( lpMapiRecipDesc pMapiRecip, DWORD dwCount );	
		
		// Destructor
		virtual ~CMapiRecip();
		
	private:	//Helper functions	
		
		DWORD CopyFromPointer( lpMapiRecipDesc pMapiRecip, DWORD dwCount );
		
		lpMapiRecipDesc GetPointer();
		
	public:		// Member functions
	
		void Flush();
			
		DWORD GetSize();
		
		DWORD CreatePointer( WORD wSize );
			
		DWORD SetPosition( DWORD dwPos );									
		
		DWORD GetRecipClass();
		
		DWORD SetRecipClass( DWORD dwNewRecipClass );
				
		CString GetName();
		
	 	DWORD SetName( CString NewName );
	 	
		CString GetAddress();
		
		DWORD SetAddress( CString NewAddress );
		
		DWORD SetEntryID ( LPVOID pEntryID, WORD wEIDSize );
		
		virtual void Serialize( CArchive& ar );
		
		CMapiRecip& operator=( CMapiRecip& b );
		
		CMapiRecip& operator+=( CMapiRecip& b );
		
		CMapiRecip& operator[]( DWORD d );

		friend CArchive& AFXAPI operator <<( CArchive& ar, CMapiRecip& recip );
		friend CArchive& AFXAPI operator >>( CArchive& ar, CMapiRecip& recip );
		
#ifdef _DEBUG
	virtual void AssertValid() const
			{ CObject::AssertValid(); }
			 
	virtual void Dump( CDumpContext& dc );	
#endif //_DEBUG                                            
};				

CArchive& AFXAPI operator <<( CArchive& ar, CMapiRecip& recip );
CArchive& AFXAPI operator >>( CArchive& ar, CMapiRecip& recip );

///////////////////////////////////////////////////////////////////////////////////////
// Provides support for MapiFile Struct.
//
class CMapiFile : public CObject
{
	DECLARE_SERIAL( CMapiFile );
	
	friend class CMapiMessage;
	
	private:
		CString			m_stringRetval;
		lpMapiFileDesc	m_pMapiFileDesc;			
		lpMapiFileDesc	m_pPosition;
		ULONG			m_ulFileCount;
		
	// Helper functions;
		DWORD CopyFromPointer( lpMapiFileDesc pMapiFile, DWORD dwCount );
			
		lpMapiFileDesc GetPointer();
		
	public:
		//	Constuctors
		CMapiFile();
		
		CMapiFile( CMapiFile& b );
		
		// Destructor
		virtual ~CMapiFile();
		
		
	//Member functions
	
		void Flush();
			
		DWORD GetSize();
		
		DWORD CreatePointer( WORD wSize );
			
		DWORD SetPosition( DWORD dwPos );
									
		DWORD GetFlags();
		
		DWORD SetFlags( DWORD dwNewFlags );
		
		DWORD SetAttachmentPosition( DWORD dwAttPos	);
				
		DWORD GetAttachmentPosition();
				
		CString GetPathName(); 
		
	 	DWORD SetPathName( CString NewPathName );

		CString GetFileName(); 
		
		DWORD SetFileName( CString NewFileName );

		virtual void Serialize( CArchive& ar );

		CMapiFile& operator=( CMapiFile& b );

		CMapiFile& operator+=( CMapiFile& b );
		
		CMapiFile& operator[]( DWORD d );
		
		friend CArchive& AFXAPI operator <<( CArchive& ar, CMapiFile& file );
        friend CArchive& AFXAPI operator >>( CArchive& ar, CMapiFile& file );
	
#ifdef _DEBUG
	virtual void AssertValid() const
			{ CObject::AssertValid(); }
	virtual void Dump( CDumpContext& dc );	
#endif
};				

CArchive& AFXAPI operator <<( CArchive& ar, CMapiFile& file );
CArchive& AFXAPI operator >>( CArchive& ar, CMapiFile& file );


///////////////////////////////////////////////////////////////////////////////
//	The class CMapiMessage is to provides easy management of the MapiMessage
//	struct.
//
class CMapiMessage : public CObject
{
	friend class CMAPI;
	
	DECLARE_SERIAL( CMapiMessage );
	
private:
	CString m_stringRetval;
	lpMapiMessage m_pMapiMessage;
	CMapiRecip    m_Orig, m_Recip;
	CMapiFile	  m_File;

// Helper Functions
	DWORD SetTimeReceived( CTime time );
	
	DWORD CreatePointer();
	
	DWORD CopyFromPointer( lpMapiMessage pMessage );

	lpMapiMessage GetPointer();

public:
// constructors.
	CMapiMessage();
	
    CMapiMessage( CMapiMessage& b );
    
    CMapiMessage( lpMapiMessage pMessage );

// destructor
	virtual ~CMapiMessage();
		
//Member functions
	void Flush();
	
	CString GetSubject();
			
	DWORD SetSubject( CString Subject );

	CString GetNoteText();
			
	DWORD SetNoteText( CString NoteText );

	CString GetMessageType();
	
	DWORD SetMessageType( CString MessageType );

	DWORD GetTimeReceived( CTime& retval );

	CString GetConversationID();
			
	DWORD SetConversationID( CString ConversationID );

	DWORD GetFlags();
		
	DWORD SetFlags( DWORD dwFlags );

	CMapiRecip& GetOriginator();

	DWORD SetOriginator( CMapiRecip& recip );
	
	CMapiRecip& GetRecips();

	DWORD SetRecips( CMapiRecip& Recip );
	
	CMapiFile& GetFiles();	

	DWORD SetFiles( CMapiFile& File );
	
	CMapiMessage& operator=( CMapiMessage& b );

	friend CArchive& AFXAPI operator <<( CArchive& ar, CMapiMessage& message );
    friend CArchive& AFXAPI operator >>( CArchive& ar, CMapiMessage& message );

#ifdef _DEBUG
	virtual void AssertValid() const
			{ CObject::AssertValid(); }
#endif
};

CArchive& AFXAPI operator <<( CArchive& ar, CMapiMessage& message );
CArchive& AFXAPI operator >>( CArchive& ar, CMapiMessage& message );

	/////////////////////////////////////////////////////////////////////////////
	//	typedef for static Mapi functions
	//
	typedef ULONG (WINAPI *tLogonProc)(ULONG, LPSTR, LPSTR, FLAGS, ULONG, LPLHANDLE);

	typedef ULONG (WINAPI *tLogoffProc)(LHANDLE, ULONG, FLAGS,ULONG);

	typedef ULONG (WINAPI *tSendMailProc)(LHANDLE, ULONG, lpMapiMessage, FLAGS,
	                                     ULONG);

	typedef ULONG (WINAPI *tSendDocumentsProc)(ULONG, LPSTR, LPSTR, LPSTR, ULONG);

	typedef ULONG (WINAPI *tFindNextProc)(LHANDLE, ULONG, LPSTR, LPSTR, FLAGS,
	                                     ULONG, LPSTR);

	typedef ULONG (WINAPI *tReadMailProc)(LHANDLE, ULONG, LPSTR, FLAGS, ULONG,
	                                     lpMapiMessage FAR *);

	typedef ULONG (WINAPI *tSaveMailProc)(LHANDLE, ULONG, lpMapiMessage, FLAGS,
	                                     ULONG, LPSTR);

	typedef ULONG (WINAPI *tDeleteMailProc)(LHANDLE, ULONG, LPSTR, FLAGS, ULONG);

	typedef ULONG (WINAPI *tFreeBufferProc)(LPVOID);

	typedef ULONG (WINAPI *tAddressProc)(LHANDLE, ULONG, LPSTR, ULONG, LPSTR,
						                ULONG, lpMapiRecipDesc, FLAGS, ULONG,
	                                    LPULONG, lpMapiRecipDesc FAR *);
  
	typedef ULONG (WINAPI *tDetailsProc)(LHANDLE, ULONG,lpMapiRecipDesc, FLAGS,
	                                    ULONG);

	typedef ULONG (WINAPI *tResolveNameProc)(LHANDLE, ULONG, LPSTR, FLAGS,
							                ULONG, lpMapiRecipDesc FAR *);
	//
	///////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
//	The class CMAPI provides all the "C" MAPI functions. Each instance of the
//	class will do a logon  and if the logon  fails all other functions are disabled.
// 
class CMAPI : public CObject
{
	DECLARE_DYNAMIC( CMAPI );
       		
public:
		
	//Constructors
	CMAPI();
	
	// Destructor;
	virtual ~CMAPI();
	
	DWORD Logon( LPCSTR lpcstrName, LPCSTR lpcstrPassword, DWORD dwFlags, HWND hWndParent );
	
	DWORD FreeBuffer( LPVOID p );

	DWORD Address(CString& Caption, DWORD dwEditFields, 
				   CString& Labels, CMapiRecip& Recip,
	 			   DWORD dwFlags, CMapiRecip& NewRecip,
	 			   CWnd *pParent = NULL );
	
	DWORD DeleteMail( CString& MessageID, CWnd *pParent = NULL );
		
	DWORD Details( CMapiRecip& Recip, DWORD dwFlags, CWnd *pParent = NULL );
	
	DWORD FindNext( CString& MessageID, CString& MessageType,
					DWORD dwFlags, CString& SeedMessageID, CWnd *pParent = NULL );
	
	DWORD ReadMail( CString& MessageID, DWORD dwFlags, CMapiMessage& Message, CWnd *pParent = NULL );
	
	DWORD ResolveName( CString& Name, DWORD dwFlags, CMapiRecip& RecipInfo, CWnd *pParent = NULL );
	
	DWORD SaveMail( CMapiMessage& MessageDetails, DWORD dwFlags, CString& MessageID, CWnd *pParent = NULL );
	
	DWORD SendMail( CMapiMessage& MessageDetails, DWORD dwFlags = 0, CWnd *pParent = NULL );
	
private:
	UINT InItMAPI();
	void DeInItMAPI();

	CWnd	*m_pParent;
	LHANDLE m_lhSession;

	static UINT m_CMAPI_InstanceCount;
	
	static HINSTANCE hinstMAPIdll;

	static tLogonProc		  lpfnMAPILogon;
	static tLogoffProc		  lpfnMAPILogoff;
	static tSendMailProc	  lpfnMAPISendMail;
	static tSendDocumentsProc lpfnMAPISendDocuments;
	static tFindNextProc	  lpfnMAPIFindNext;
	static tReadMailProc	  lpfnMAPIReadMail;
	static tSaveMailProc	  lpfnMAPISaveMail;
	static tDeleteMailProc	  lpfnMAPIDeleteMail;
	static tAddressProc		  lpfnMAPIAddress;
	static tResolveNameProc	  lpfnMAPIResolveName;
	static tDetailsProc		  lpfnMAPIDetails;
	static tFreeBufferProc	  lpfnMAPIFreeBuffer;

	
#ifdef _DEBUG
	virtual void AssertValid() const
				{ CObject::AssertValid(); }
#endif
  		
};

#endif // __CLASMAPI_H



