#include "stdafx.h"
#include "util.h"
#include "cwaitcur.h"
#include "resource.h"
#include "const.hxx"
#include "clasmapi.h"
#include "mailexts.h"
#include "command.h"
#include "dbsync.h"


//static const char BASED_CODE SZ_BLANK[] = "";

//local stuff to this file
BOOL DoDBSync(HWND hWnd, CMAPI& mailSession, CMapiMessage& mapiMessage, CString& strMsgID, LPCSTR lpcstrMessageType, BOOL bSaveLocalCopy, LPCSTR lpcstrDSN);
BOOL RemoveConfimFile(LPCSTR lpcstrNote, LPCSTR lpcstrSentBy);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//processess custom Mail messages from Microsoft Mail
//the .INI file settings that are refered to here (defined in const.hxx) are also defined in the DBSync modules in the
//access database application.
//It is necessary (if you want this to work) that these settings are the same between applications.

//in particular, the following settings are important
/*
	[AttachedData]
	DataDB= <the database with all of the tables (including the dbx_* tables) in it>
	
	[DBSync]
	ShowErrorMessages= 1 or 0 (yes or no)
	LocalStoragePath= <where are the .DBX files stored? >
	DSNRemote=<has to match the [AttachedData], DataDB= file>
	DSNToCorp = <SQL server datasource>
	SQLDatabaseName= <name of the SQL server database>

*/

//some attempt is made to imform the user of any errors in the .INI file, but this is probably not foolproof in this demo
//standardizing names may help, or in some cases, the datasources could be created.
//however, the user should not be required to select or create a proper datasource, as this may be a process that is left unattended
//especially on the SQL server side.

//additionally, you will want to be sure and create a database user in SQL server with insert, update, select, and delete priviledges
//that has a user id and password that matches the uid and pwd settings in the const.hxx file (in this case UID=dbsync;PWD=dbsync)

//also, just one more warning, this process has been tested on a NON-case sensitive SQL server. (ONLY)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

long WINAPI  DBX_EXPORT Command(PARAMBLK FAR* pparamblk)
{
	CString strMsgID = pparamblk->lpMessageIDList;
  	CString strMessageType;
  	HCURSOR hWaitCur = NULL;
  	
  	AfxLockTempMaps();
  	
  	TRY
  	{
	  	ULONG ulResult = SUCCESS_SUCCESS;
	  	char szRemoteDB[256] = "";
	  	char szDBQ[256] = "";
	  	char szDSN[64]="";	  	
	  	
	  	
	  	BOOL bSaveLocalCopy = FALSE; //save a local "copy" of each transaction? (to SQL Server ONLY)
	  	
		CMAPI mailSession; //loads the mapi.dll and finds all the entry points
		//note that the destructor will do a logoff if a vaild session has been established
				
		//try to get the current session, otherwise, put up a dialog
		if(SUCCESS_SUCCESS != (ulResult = mailSession.Logon("", "", MAPI_LOGON_UI, pparamblk->hwndMail)))
		{
			goto Command_exit;
		}
		
		hWaitCur = ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
		
		// read the message, PEEK only
		CMapiMessage mapiMessage;
		if(SUCCESS_SUCCESS != (ulResult = mailSession.ReadMail(strMsgID, 
			MAPI_PEEK | MAPI_GUARANTEE_FIFO, mapiMessage)))
		{
			char szMsg[64];
			wsprintf(szMsg, "Error reading Mail.\nMapi Error %li", ulResult);
			AfxMessageBox(szMsg, MB_ICONHAND | MB_TASKMODAL);
			goto Command_exit;
		}
				    		
		strMessageType = mapiMessage.GetMessageType(); //message type one of the following 3
		//1. DBSync (from SQL (or Corp) to the remote databases)
		//2. DBSyncToCorp  (from the remote database(s) to SQL Server (or corp))
		//3. Confirmation (delete the local file from the DBSync local storage path
		
		TRACE1("Message Type: '%s'\n", (LPCSTR)strMessageType);
				
		if(0 == strMessageType.CompareNoCase(SZ_DBSYNC_MSGTYPE)){			
			
			//this is a dbsync message, from the the main (SQL server) database
			TRACE("DBSync message\n");
			
			//make SURE that the attached "data" database is the same as the Jet DBQ= value			
			GetPrivateProfileString(SZ_INISEC_ATTACHEDDATA, SZ_INIKEY_DATADB, "", szRemoteDB, sizeof(szRemoteDB), SZ_INIFILE);
			TRACE1("[AttachedData], DataDB= szRemoteDB: '%s'\n", szRemoteDB);
			
			if(!FileExists(szRemoteDB))
			{
				AfxMessageBox("[AttachedData] DataDB= file is missing.\nCannot continue.", MB_ICONINFORMATION); 				
			}
			else
			{			
				//[DBSync], DSNRemote= 
				GetPrivateProfileString(SZ_INISEC_DBSYNC, SZ_INIKEY_DBSYNCDSN, "", szDSN, sizeof(szDSN), SZ_INIFILE);
				TRACE1("[DBSync], DSNRemote= szDSN:  '%s'\n", szDSN);
				if(lstrlen(szDSN) > 0)
				{
					GetPrivateProfileString(szDSN, "DBQ", "", szDBQ, sizeof(szDBQ), "odbc.ini");
						if(0 != lstrcmpi(szRemoteDB, szDBQ))
						{
							AfxMessageBox("DSNRemote= datasource DBQ file is not the same as [AttachedData], DataDB= in dbsync.ini");							
						}
						else				
						{
							//Do the apply, everything is right							
							bSaveLocalCopy = FALSE;
							DoDBSync(pparamblk->hwndMail, mailSession, mapiMessage, strMsgID, strMessageType, bSaveLocalCopy, szDSN);
						}
				}
				else
				{
					//there is no DSN= entry at all					
					AfxMessageBox("DSN= datasource is missing from dbsync.ini");												
				}				
			
			}
			
		}
		else if(0 == strMessageType.CompareNoCase(SZ_DBSYNCTOCORP_MSGTYPE)){			
			//this is a dbsync to corp (to SQL Server) message			
			TRACE("DBSyncToCorp message\n");
			
			GetPrivateProfileString(SZ_INISEC_DBSYNC, SZ_INIKEY_DBSYNCDSNTOCORP, "", szDSN, sizeof(szDSN), SZ_INIFILE);
			
			TRACE1("szDSN:  '%s'\n", szDSN);
			
			bSaveLocalCopy = TRUE;
			DoDBSync(pparamblk->hwndMail, mailSession, mapiMessage, strMsgID, strMessageType, bSaveLocalCopy, szDSN);
		}
		else if(0 == strMessageType.CompareNoCase(SZ_DBSYNCCONFIRM_MSGTYPE)){			
		
			//this is a dbsync confirmation message			
			TRACE("DBSyncConfirm message\n");
			
			CString strNote = mapiMessage.GetNoteText(); //should be name of the file
			
			CMapiRecip* pSentBy = new CMapiRecip;
			*pSentBy = mapiMessage.GetOriginator();
			CString strSentBy = pSentBy->GetName();
			delete pSentBy;
			
			TRACE2("Command():  strNote == %s\nstrSentBy == %s\n", (LPCSTR)strNote, (LPCSTR)strSentBy);
			
			if(RemoveConfimFile(strNote, strSentBy))
				mailSession.DeleteMail(strMsgID);				
			
			
		}
		
		//don't do anything, just drop out
		
	}
	CATCH(CMapiException, e)
	{		
		AfxMessageBox(e->m_strError, MB_TASKMODAL | MB_ICONHAND);	
	}
	AND_CATCH(CException, e)
	{
		AfxMessageBox(IDS_E_UNDEFINEDEXCEPTION, MB_TASKMODAL | MB_ICONHAND);
	}
	END_CATCH
		
	strMsgID.Empty();
	strMessageType.Empty();
	
	if(hWaitCur)
		::SetCursor(hWaitCur);
		
	Command_exit:
	
	AfxUnlockTempMaps();
	
		
	#ifdef _DEBUG			
		CMemoryState memState;
		memState.DumpAllObjectsSince();		
	#endif		//_DEBUG

		
	return 0L;


}



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  		Function DoDBSync()		
//			
//
//          parameters:
//				HWND hWnd													Mail main window
//				CMAPI mailSession											reference to a valid CMAPI object
//				CMapiMessage mapiMessage 							reference to a CMapiMessage object
//				CString strMsgID												 reference to the Mail messageid string
//				LPCSTR lpcstrMessageType							the type of this message
//				BOOL bSaveLocalCopy										true specifies that a local copy of the transactions should be saved
//				LPCSTR lpcstrDSN										the datasource name
//
//						
//			
//			Reads the message (represented by strMsgID), saves the file(s) (CMapiMessage.GetFiles()); there is probably only one
//			and calls CDBSync::ApplyTransactionFile for each file,
//			notice that the Datasource name has already been determined (based on the message type) by the caller, (Command())
//
//			returns TRUE (success) or FALSE (fail)
//
//
BOOL DoDBSync(HWND hWnd, CMAPI& mailSession, CMapiMessage& mapiMessage, 
	CString& strMsgID, LPCSTR lpcstrMessageType, BOOL bSaveLocalCopy, LPCSTR lpcstrDSN)
{
	char szMsgType[64];
	char szDestPath[256];
	
	BOOL bSuccess = FALSE;
		
	memset(szDestPath, 0, sizeof(szDestPath));
	strncpy(szMsgType, mapiMessage.GetMessageType(), sizeof(szMsgType));
	
	CMapiRecip* pSentBy = NULL;
	CString strSubject;
	
	TRY
  	{
  		pSentBy = new CMapiRecip;
		*pSentBy = mapiMessage.GetOriginator();
		
        //get the subject		
		strSubject = mapiMessage.GetSubject(); //subject is always "DBSync"
			
		//get the files object
		CMapiFile mapiFiles;
		mapiFiles = mapiMessage.GetFiles();
		
		DWORD dwSize, dwPos;
		dwSize = mapiFiles.GetSize(); //how many files (should be 1)
					
		CString strPathName, strOrigFileTitle, strSentBy;
		for(dwPos = 0; dwPos < dwSize; ++dwPos){
			
			strPathName = mapiFiles[dwPos].GetPathName();									
			strOrigFileTitle = mapiFiles[dwPos].GetFileName();									
			TRACE2("DoDBSync() -> Path Name == %s\tFileName == %s\n", (LPCSTR)strPathName, (LPCSTR)strOrigFileTitle);
			strSentBy = pSentBy->GetName();
									
			bSuccess = CDBSync::ApplyTransactionFile(hWnd, strSentBy, strPathName, strOrigFileTitle, lpcstrDSN, bSaveLocalCopy);							
							
		}
		
		//only delete the message if the apply was successful
		if(bSuccess)
			mailSession.DeleteMail(strMsgID);
			
		if(FileExists(strPathName))
			remove(strPathName);
		
				
	}
	CATCH(CDBSyncException, e)
	{
		if(!e->m_strError.IsEmpty())
			AfxMessageBox(e->m_strError, MB_ICONINFORMATION | MB_TASKMODAL);
	}
	AND_CATCH(CMapiException, e)
	{
		strSubject.Empty();
		AfxMessageBox(IDS_E_MAPIEXCEPTION, MB_TASKMODAL | MB_ICONHAND);	
	}
	AND_CATCH(CException, e)
	{
		AfxMessageBox(IDS_E_UNDEFINEDEXCEPTION, MB_TASKMODAL | MB_ICONHAND);
	}
	END_CATCH
	
	if(pSentBy)
		delete pSentBy;
	
	return (bSuccess);

}




////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//			Function RemoveConfirmFile()
//
//         params
//			LPCSTR lpcstrNote									the note text of the message
//																				which is actually the file title
//          LPCSTR lpcstrSentBy                                Sender name
//
//
//
//			deletes the local file (confirmation) associated with an earlier DBSync message
//			the lpcstrNote contains the actual file name (with the subdirectory if this is going to corporate (SQL Server))
// 			<usernumber>\filename.dbx
//
//
//
//         also writes a LOG ENTRY for debugging purposes
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL RemoveConfimFile(LPCSTR lpcstrNote, LPCSTR lpcstrSentBy)
{
	char szPath[256], szFile[16];
	
	memset(szFile, 0, sizeof(szFile));
	
	
	//find out where the local files are stored
	::GetPrivateProfileString(SZ_INISEC_DBSYNC, SZ_INIKEY_DBSYNCLOCALCACHE, "", 
		szPath, sizeof(szPath), SZ_INIFILE);	
		
	TRACE1("[DBSync], LocalStoragePath == %s\n", szPath);
	
	
	//add the filename to the path
	int nLen = lstrlen(szPath);
	if(nLen && szPath[nLen - 1] != '\\')
		lstrcat(szPath, "\\");
		
	if(0 == nLen)
		return FALSE;
		
	lstrcat(szPath, lpcstrNote);
	AnsiUpper(szPath);
	nLen = lstrlen(szPath);
		
	while(--nLen){
		if(szPath[nLen] == ' ' || szPath[nLen] == '\r' || szPath[nLen] == '\n')
			szPath[nLen] = '\0';
	}
	
	//write a log entry for each confirmation message
	//[Sent By User]
	//Path name of file = Date and time
	CString strNow;
	CTime t = CTime::GetCurrentTime();
	strNow = FormatCTime(&t);
	::WritePrivateProfileString(lpcstrSentBy, szPath, strNow, SZ_DBSYNCCONFIRMEDLOG);
	
	if(!FileExists(szPath))
	{
		TRACE1("File to be deleted (%s) does not exist.\n", szPath);
		return TRUE;
	}
		
	//delete it
	if(0 == remove(szPath))
	{
		TRACE1("Successfully deleted file:  %s\n", szPath);
		return TRUE;
	}
	else
	{
		TRACE1("Error deleting file:  %s\n", szPath);
		return FALSE;
	}
		
	
	
}
