#include "stdafx.h"
#include "util.h"
#include "cwaitcur.h"
#include "resource.h"
#include "const.hxx"
#include "userutil.h"
#include "clasmapi.h"
#include <lzexpand.h> //for LZCopy






////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																																										
//		Function DBSyncFileSend()
//		parameters:
//			LPSTR lpszMessageType == 			  the custom message type (IPM.DBSync.DBSync, ...)					
//          LPSTR lpszSubject == 						subject of the message
//          LPSTR lpszMessageText  == 			   the text of the message (should be at least 1 space if a file is attached)
//			LPSTR lpszRecipient ==                     the SINGLE resolvable name (actual email name)
//			LPSTR lpszFileName == 			         the optional file name (can be null or "")
//			LONG lFlags ==								   a bitmask of flags (currently  SMF_SHOWDIALOG or SMF_NOMESSAGES)
//																	   these may be Or'ed together 
//
//		Sends a mail message and (optionally) a file
//		
//
//      returns TRUE (success) or FALSE (failure)
//
//
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//local version
int Handle_DBSyncFileSend(LPSTR lpszMessageType, LPSTR lpszSubject, LPSTR lpszMessageText, 
	LPSTR lpszRecipient, LPSTR lpszFileName, LONG lFlags);

int WINAPI DBX_EXPORT DBSyncFileSend(LPSTR lpszMessageType, LPSTR lpszSubject, LPSTR lpszMessageText, 
	LPSTR lpszRecipient, LPSTR lpszFileName, LONG lFlags)
{
	AfxLockTempMaps();
	int rc = Handle_DBSyncFileSend(lpszMessageType, lpszSubject, lpszMessageText, lpszRecipient, lpszFileName, lFlags);
	AfxUnlockTempMaps();
	return rc;
}
	
int Handle_DBSyncFileSend(LPSTR lpszMessageType, LPSTR lpszSubject, LPSTR lpszMessageText, 
	LPSTR lpszRecipient, LPSTR lpszFileName, LONG lFlags)
{

	int fShowDialog = FALSE;
	int fCompressFile = FALSE;
	int fNoMessages = FALSE;
	DWORD dwResult = SUCCESS_SUCCESS;	
	BOOL bZipSuccess = FALSE;	
	CMAPI* pMailSession = NULL;	
	
	fShowDialog = (SMF_SHOWDIALOG == (lFlags & SMF_SHOWDIALOG)); // show the mail dialog every time?
	//note that compression is NOT supported in this version, this flag is provided for future compatibility
	fCompressFile = (SMF_COMPRESSFILE == (lFlags & SMF_COMPRESSFILE)); // zip the file?
	
	fNoMessages = (SMF_NOMESSAGES == (lFlags & SMF_NOMESSAGES)); // don't display messages	
 	
 	TRY
	{
		pMailSession = new CMAPI;
		CMapiMessage mapiMessage;
		CMapiFile mapiFile;
		CMapiRecip mapiRecip;
		
		pMailSession->Logon("", "", MAPI_LOGON_UI, ::GetActiveWindow());			
			
		CWaitCursor wait;
							
		//set the message note text (just space for the file)
		CString strNoteText;
		strNoteText += lpszMessageText;
		mapiMessage.SetNoteText(strNoteText);		
		//only set the message type if it is NOT null and longer than 0 characters
		if(NULL != lpszMessageType && lstrlen(lpszMessageType) > 0){
			CString strMessageType(lpszMessageType);
			mapiMessage.SetMessageType(strMessageType);
		}			
		else
			fCompressFile = FALSE;	//only compress files with a custom message type
		
		CString strSubject(lpszSubject);
		mapiMessage.SetSubject(strSubject);
		
		// need to resolve the name/number
		CString strName(lpszRecipient);
		if(SUCCESS_SUCCESS == pMailSession->ResolveName(strName, MAPI_DIALOG, mapiRecip))
			mapiMessage.SetRecips(mapiRecip); //set the recipients
		else
			fShowDialog = TRUE;		
		
		//add the file attachment
		if(lpszFileName && (lstrlen(lpszFileName) > 0) && FileExists(lpszFileName)){
			
			mapiFile.CreatePointer(1);
			mapiFile[0].SetAttachmentPosition(0);
			mapiFile[0].SetPathName(lpszFileName);
			mapiFile[0].SetFlags(0);
			
			if(SUCCESS_SUCCESS != (dwResult = mapiMessage.SetFiles(mapiFile))){
				if(!fNoMessages)
					AfxMessageBox(IDS_E_MISSINGATTACHMENT, MB_ICONHAND);
				
				delete pMailSession;
				
				return FALSE;		
			}
		}
			     		
		
		DWORD dwFlags;
		dwFlags = fShowDialog ? MAPI_DIALOG : 0;
			
		dwResult = pMailSession->SendMail(mapiMessage, dwFlags);
		
		BOOL bReturn = (SUCCESS_SUCCESS == dwResult);		
		
		delete pMailSession;
		
		
		return bReturn;
		
		
	}
	CATCH(CMapiException, e)
	{
		 AfxMessageBox(e->m_strError, MB_TASKMODAL | MB_ICONHAND);			 
	}
	AND_CATCH(CException, e)
	{
	   AfxMessageBox(IDS_E_UNDEFINEDEXCEPTION, MB_SYSTEMMODAL | MB_ICONHAND);		
	}
	END_CATCH
	
	if(pMailSession)
		delete pMailSession;
	
		
	#ifdef _DEBUG			
		CMemoryState memState;
		memState.DumpAllObjectsSince();		
	#endif		//_DEBUG

	
	return FALSE;
	
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																																										
//		Function DBSyncCopyFile( <source file name>, <destination file name>)
//
//
//		copies a file (lpcstrSource) to a destination file lpcstrDest
//		the destination file WILL BE OVERWRITTEN UNCONDITIONALLY
//
//      returns the number of bytes copied (greater than zero) or -1 (error) or 0 (source file does not exist)
//
//
//		uses the lzexpand.dll for copying
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _WIN32
long WINAPI DBX_EXPORT DBSyncCopyFile(LPCSTR lpcstrSource, LPCSTR lpcstrDest)
{

	CWaitCursor wait;
	
	long lBytes = 0;
	OFSTRUCT ofStrSrc, ofStrDest;
	HFILE hfSrcFile = HFILE_ERROR;
	HFILE hfDstFile = HFILE_ERROR;

	if(!FileExists(lpcstrSource))
		return (0L);

	if(HFILE_ERROR == (hfSrcFile = LZOpenFile(lpcstrSource, &ofStrSrc, OF_READ)))
		return -1L;
	
	if(HFILE_ERROR == (hfDstFile = LZOpenFile(lpcstrDest, &ofStrDest, OF_CREATE))){
		LZClose(hfSrcFile);			
		return -1L;	
	}	

	lBytes = LZCopy(hfSrcFile, hfDstFile);

	LZClose(hfSrcFile);
	LZClose(hfDstFile);
	
	return (lBytes);

}
#else
long WINAPI DBX_EXPORT DBSyncCopyFile(LPCSTR lpcstrSource, LPCSTR lpcstrDest)
{
	return CopyFile(lpcstrSource, lpcstrDest, FALSE);
}
#endif //!_WIN32
