#include "stdafx.h"
#include "util.h"
#include <dos.h>
#include <math.h>
#include <string.h>

static char sDate[4], s1159[4], s2359[4], sTime[4];
static int iDate, iTime, bDateLeadingZeros, iTLZero, bCentury;
static char szDateFmt[16], szTimeFmt[16];


#ifdef _WIN32

	#include <io.h>
	BOOL FileExists(const char* lpcstrPathName)
	{
		static struct _finddata_t fileinfo;
		long lHandle;
		if(-1 != (lHandle = _findfirst((char*)lpcstrPathName, &fileinfo)))
			_findclose(lHandle);
			
		return(-1 != lHandle);
	}
#else //!_WIN32
	BOOL FileExists (const char * lpcstrPathName)
	{
		static struct _find_t fileinfo;
		return(0 == _dos_findfirst(lpcstrPathName, _A_NORMAL, &fileinfo));
	}
#endif //_WIN32

//removes quotes from a string, 
//the quotes (ascii 34) must be the first and last character and the string must not contain
//the magic delimiter (cDelim) anywhere in the string
void StripQuotes(LPSTR szBuf, char cDelim)
{
	if(strchr(szBuf, cDelim))
		return;

	UINT nLen = lstrlen(szBuf);
	if((nLen == 0) || (szBuf[0] != 34) && (szBuf[nLen - 1] != 34))
		return;

	memmove(szBuf, (szBuf + 1), nLen - 1);
	szBuf[nLen -2] = '\0';

}


void QuoteWrap(LPSTR szBuf, char chDelim)
{
	#define CHARSMAX 300
	static char szTemp[CHARSMAX + 16];
	char szQuote[2] = "\"";

	if(0 == lstrlen(szBuf))
    	return ;
	if(NULL == strchr(szBuf, chDelim))
		return ;

	if(lstrlen(szBuf) > CHARSMAX)
		szBuf[CHARSMAX] = '\0';

	if(34 != szBuf[0]){
		memset(szTemp,0,sizeof(szTemp));
		lstrcat(szTemp, szQuote);
		lstrcat(szTemp, szBuf);
		if(34 != szTemp[lstrlen(szTemp)-1])
			lstrcat(szTemp, szQuote);

		lstrcpy(szBuf, szTemp);

        return ;

	}

	if(34 != szBuf[lstrlen(szBuf)-1])
    	lstrcat(szBuf, szQuote);
	return ;
}

int StringToStringArray(LPCSTR lpsz, CStringArray* pstrArray, char cListSep)
{
	LPSTR pszStart, pszEnd, pszNext;
	int len, nCount;
	
	len = nCount = 0;
		
	pstrArray->FreeExtra();
	pstrArray->RemoveAll();
	
	if(NULL == lpsz || 0 == lstrlen(lpsz))
		return 0;
	
	pszStart = (LPSTR)lpsz;
	pszEnd = strchr(pszStart, cListSep);
	if(NULL == pszEnd){
		pstrArray->Add((LPCSTR)lpsz);
		return 1;
	}
	
	char* szTemp = new char[lstrlen(lpsz) + 1];
	lstrcpy(szTemp, lpsz);
	pszStart = szTemp;
		
	while(pszStart){

		if(pszEnd)
			*pszEnd = '\0';

		/* check for embedded delimiters ... ONLY ["????,????] NOT ["????,????"] */
		if( (34 == pszStart[0]) && ((1 == lstrlen(pszStart)) || (34 != pszStart[lstrlen(pszStart) - 1])) ){
			pszNext = pszEnd;
			*pszNext = cListSep; //reset the end pointer
			
			pszNext = strchr(++pszNext, cListSep);
			
			if(NULL == pszNext){
				/* this is the last field in the line ("1,2,3"\0) */
				len = lstrlen(pszStart);
				if(len > 1){
					if(34 == pszStart[len - 1]) // last field, with a quote
						pszEnd = &pszStart[len];
				}
			}
				
			while(pszNext){
				if(pszNext[-1] == 34){
                		pszEnd = pszNext;
					*pszNext = '\0';
					break;
				}
				
				if(NULL == (pszNext = strchr(++pszNext, cListSep)) ){
               	 	/* this is the last field in the line ("1,2,3"\0) */
					len = lstrlen(pszStart);
					if(len > 1){
						if(34 == pszStart[len - 1]) // last field, with a quote
							pszEnd = &pszStart[len];
					}
               	 	break ;
               	 }
			}
			
		}
          
          RemoveQuotes(pszStart);
          
          pstrArray->Add((LPCSTR)pszStart);
		        
		pszStart = ++pszEnd;

		if(!lstrlen(pszStart))
			break;
		
		++nCount;		
		
		pszEnd = strchr(pszStart, cListSep);

	}
	
	delete szTemp;	

	return (pstrArray->GetSize());

}

//unconditionally removes quotes both szBuf[0] and szBuf[strlen(szBuf)-1] MUST be chr$(34)'s
void RemoveQuotes(LPSTR lpsz)
{
	int nLen = lstrlen(lpsz);
	if(nLen == 0)
		return;
		
	if((lpsz[0] != 34) || (lpsz[nLen - 1] != 34))
		return;

	memmove(lpsz, (lpsz + 1), nLen - 1);
	lpsz[nLen -2] = '\0';

}

void trim_space(LPSTR sz)
{
	int len;
	len = lstrlen(sz);
	if(0 == len)
		return;

	do{
		if(' ' == sz[--len])
			sz[len] = '\0';
		else
			break;        
	}while(len > 0);


}


//returns a pointer to a static buffer
LPCSTR FormatCTime(CTime *pTime, LPCSTR lpcstrFormat)
{
    int nDay, nMonth, nYear, nHour, nMin;
	static char szTime[128], szTempDate[64], szTempTime[64], szAMPM[16];
	memset(szTime, 0, sizeof(szTime));
	memset(szTempDate, 0, sizeof(szTempDate));
	memset(szTempTime, 0, sizeof(szTempTime));
	
	if(*pTime == 0)
		return szTime;
	
	GetIntlSettings();
	nDay = pTime->GetDay();
	nMonth = pTime->GetMonth();
	nYear = pTime->GetYear();
	nHour = pTime->GetHour();
	nMin = pTime->GetMinute();	
	
	static char szDay[4], szMonth[4], szYear[8], szHour[4], szMin[4];
	
	BOOL bSQLFmt = (0 == lstrcmpi(SZ_SQL_DATE, lpcstrFormat) || 0 == lstrcmpi(SZ_SQL_TIMESTAMP, lpcstrFormat));
	
	if(!bSQLFmt && 0 == iTime){
		lstrcpy(szAMPM, " ");
		if(nHour > 12){
			nHour -= 12;
			lstrcat(szAMPM, s2359);
		}
		else{				
			lstrcat(szAMPM, s1159);	
		}
		
	}
	else
		lstrcpy(szAMPM, "");
		
	//bSQLFmt always wants century
	
	if((bSQLFmt || bCentury) && nYear < 1900)
		nYear += 1900;
	else{
		if(!bSQLFmt && nYear > 1900)
			nYear -= 1900;
	}
		
	wsprintf(szYear, "%i", nYear);
	
		if(bSQLFmt || iTLZero){//time leading zeros
			if(nHour < 10)
				wsprintf(szHour, "0%i", nHour);
			else
				wsprintf(szHour, "%i", nHour);
					
		}
		else{
			wsprintf(szHour, "%i", nHour);	
		}
		
		if(nMin < 10)
			wsprintf(szMin, "0%i", nMin);
		else
			wsprintf(szMin, "%i", nMin);
		
		if(bSQLFmt || bDateLeadingZeros){//date leading zeros
			if(nDay < 10)
				wsprintf(szDay, "0%i", nDay);
			else
				wsprintf(szDay, "%i", nDay);
			
			if(nMonth < 10)
				wsprintf(szMonth, "0%i", nMonth);
			else
				wsprintf(szMonth, "%i", nMonth);		
		}
		else{  
			wsprintf(szMonth, "%i", nMonth);		
		     wsprintf(szDay, "%i", nDay);
		}
		
		if(bSQLFmt){
			
			wsprintf(szTempDate, "%s-%s-%s", szYear, szMonth, szDay);
			if(0 == lstrcmpi(SZ_SQL_TIMESTAMP, lpcstrFormat))
				wsprintf(szTempTime, "%s:%s:00", szHour, szMin);	
		
		}
		else if(0 == lstrcmpi(SZ_GENERAL_DATE, lpcstrFormat) || 0 == lstrcmpi(SZ_SHORT_DATE, lpcstrFormat)){
			if(0 == iDate)//m/d/y
				wsprintf(szTempDate, szDateFmt, szMonth, szDay, szYear);
			else if(1 == iDate)//d/m/y
				wsprintf(szTempDate, szDateFmt, szDay, szMonth, szYear);
			else //y/m/d
			      wsprintf(szTempDate, szDateFmt, szYear, szMonth, szDay);		
			
			if(nHour > 0){
				if(0 == lstrcmpi(SZ_GENERAL_DATE, lpcstrFormat)){	      
					wsprintf(szTempTime, szTimeFmt, szHour, szMin);		
					lstrcat(szTempTime, szAMPM);
				}
			}
			
		}
	
	
	
	if(lstrlen(szTempTime) > 0)
		wsprintf(szTime, "%s %s", szTempDate, szTempTime);
	else
	     lstrcpy(szTime, szTempDate);
	     
	return szTime;
}

void GetIntlSettings(BOOL bReRead)
{
	
	static char BASED_CODE szIntl[] = "Intl";
	static char sShortDate[32];
	static BOOL bRead = FALSE;
	
	if(bReRead || !bRead){
	
		::GetProfileString(szIntl, "sDate", "/", sDate, sizeof(sDate));
		::GetProfileString(szIntl, "sTime", ":", sTime, sizeof(sTime));
		::GetProfileString(szIntl, "s1159", "AM", s1159, sizeof(s1159));
		::GetProfileString(szIntl, "s2359", "PM", s2359, sizeof(s2359));	  
		::GetProfileString(szIntl, "sShortDate", "m/d/yy", sShortDate, sizeof(sShortDate));	  
		
		iTLZero = ::GetProfileInt(szIntl, "iTLZero", 0);
		bDateLeadingZeros = ::GetProfileInt(szIntl, "bDateLeadingZeros", 0);
		iTime = ::GetProfileInt(szIntl, "iTime", 0);
		iDate = ::GetProfileInt(szIntl, "iDate", 0);
		
		AnsiLower(sShortDate);
		bCentury = (NULL != strstr(sShortDate, "yyyy"));
		bDateLeadingZeros = (NULL != strstr(sShortDate, "mm") && NULL != strstr(sShortDate, "dd"));
		
		lstrcpy(szDateFmt, "%s");
		lstrcat(szDateFmt, sDate);
		lstrcat(szDateFmt, "%s");
		lstrcat(szDateFmt, sDate);
		lstrcat(szDateFmt, "%s");
		
		
		lstrcpy(szTimeFmt, "%s");
		lstrcat(szTimeFmt, sTime);
		lstrcat(szTimeFmt, "%s");		
		
		bRead = TRUE;
	}
		
			
}

//converts a double to a string (works in a .DLL)
//returns the string in the buffer lpsz
int DoubleToString(double dblFloat, LPSTR lpsz, int nBufSize)
{	
     		
	#define LOG_OF_2                0.301029995

   	static char szT[256];
   	static int nExponent;
   	static double dblInt;
   	static double dblFraction;
     static double dblTempNumber;
   	static double dblMantissa;
     static unsigned long ulTempFraction;     
   	static int nDesired10Power;
   	   	
	szT[0] = '\0';
	
	
   	//Get the fractional part and the integer part of the float

   	dblFraction = modf(dblFloat, &dblInt);
   
   	// If exponent is too big or too small, 
   	// then show the float in scientific notation
   	if ((fabs(dblFloat) > 1E10 || fabs(dblFloat) < 1E-10) && dblFloat){
     	// Convert the exponent from base 2 to base 10
      	dblMantissa = frexp(dblFloat, &nExponent);
      	nDesired10Power = (int)(LOG_OF_2 * nExponent);
      	dblTempNumber = dblMantissa * pow(2, nExponent) / pow(10, nDesired10Power);
      	if (dblFloat<0)
          	dblTempNumber = -dblTempNumber;
      	dblFraction = modf(dblTempNumber, &dblInt);
   	}

   	dblFraction = fabs(dblFraction) * 1000;
   	ulTempFraction = (unsigned long)dblFraction;

   
   	// round up if the discarded portion
   	// of the fraction is greater or equal to 0.5.
   	if (dblFraction - (double)ulTempFraction >= 0.5)
     	ulTempFraction++;

	// don't display the exponent if it is zero
 	if ((fabs(dblFloat) > 1E10 || fabs(dblFloat) < 1E-10) && dblFloat)
     	wsprintf(szT, "%ld.%06luE%d", (long)dblInt, ulTempFraction, nDesired10Power);
   	else // The float can be shown without scientific notation     
		wsprintf(szT, "%ld.%06lu", (long)dblInt, ulTempFraction);
      
     int nLen = lstrlen(szT);
     int nDecPos = nLen - 4;
     
     //trim off the redundant zeros
     if(nLen > 3){
		     	
     	while(nLen > nDecPos){
     		if('0' == szT[--nLen])
     			szT[nLen] = '\0';
     		else
     			break;
     	
     	}	
     }
     
     
     lstrcpyn(lpsz, szT, nBufSize);
     return lstrlen(lpsz);

}


/* constructs a CTime object from a string */
/* the string should be in the format m/d/yy h:mm[AM/PM] */
/* 12 hour time , never longer that mm/dd/yy hh:mm AM (18) characters */
BOOL StringToCTime(LPCSTR lpsz, CTime* pTime)
{
	GetIntlSettings();
	
	tm t1;
	static char szBuf[32], szTime[16], szMin[8], szAMPM[8], szDatePart1[8], szDatePart2[8], szDatePart3[8];
	memset(&t1, 0, sizeof(tm));
	_tzset();
    LPSTR p ;	
	memset(szTime, 0, sizeof(szTime));
	*pTime = 0; //assume failure

	strncpy(szBuf, lpsz, sizeof(szBuf)); //save the date
	
	if(lstrlen(szBuf) < 6)
	    	return FALSE;
	//get the time
    	LPSTR pszTime = strchr(szBuf, ' '); //always a space after the date
    	if(pszTime){
    		*pszTime = '\0';
		strncpy(szTime, ++pszTime, sizeof(szTime)); //save the time		
	}

    	//first element
	if(NULL == (p = strtok(szBuf, sDate)))
		return FALSE;
	strncpy(szDatePart1, p, sizeof(szDatePart1));
	if(NULL == (p = strtok(NULL, sDate)))
		return FALSE;
	strncpy(szDatePart2, p, sizeof(szDatePart2));
	if(NULL == (p = strtok(NULL, sDate)))
		return FALSE;
	strncpy(szDatePart3, p, sizeof(szDatePart3));
		
	LPSTR pszD1, pszD2, pszD3;
	pszD1 = szDatePart1;
	pszD2 = szDatePart2;
	pszD3 = szDatePart3;
	
	if('0' == szDatePart1[0])
		++pszD1;
	if('0' == szDatePart2[0])
		++pszD2;
	if('0' == szDatePart3[0])
		++pszD3;
	
	switch(iDate){
		case 0: //m/d/y
		{
			t1.tm_mon = atoi(pszD1);		
			t1.tm_mday = atoi(pszD2);
			t1.tm_year = atoi(pszD3);
			break;
		}
		case 1: //d/m/y
		{
			t1.tm_mday = atoi(pszD1);
			t1.tm_mon = atoi(pszD2);	
			t1.tm_year = atoi(pszD3);
			break;
		}
		case 2: //y/m/d
		{
	     	t1.tm_year = atoi(pszD1);
	     	t1.tm_mon = atoi(pszD2);
	     	t1.tm_mday = atoi(pszD3);			
	     	break;
	     }
	     default:
	     	return -1;		
	}
	
		     	
       	if(0 == lstrlen(szTime))
    		t1.tm_hour = 0; 
    	else{
          
		//hour
		p = strchr(szTime, sTime[0]);
		if(!p)
			t1.tm_hour = 0; 
	    else{
			*p = '\0';
			t1.tm_hour = atoi(szTime);
	
			//min
			strncpy(szMin, ++p, sizeof(szMin));
				
			if(lstrlen(szMin) > 2){
				
				p = strchr(szMin, ' ');
				if(p)
					strncpy(szAMPM, ++p, sizeof(szAMPM));
				else
					strncpy(szAMPM, (char *)(szMin + 2), sizeof(szAMPM)); // get am or pm
					
				szMin[2] = '\0';
	    			
		    		AnsiUpper(szAMPM);
		    			    					
				if(0 == lstrcmpi(szAMPM, s2359) && (t1.tm_hour < 12))
					t1.tm_hour += 12;
				else if(0 == lstrcmpi(szAMPM, s1159) && (t1.tm_hour == 12))
					t1.tm_hour = 0; // 12:00AM
	          
		    	}
			
			t1.tm_min = atoi(szMin);
		}
	}

	t1.tm_isdst = -1;
	
	if(t1.tm_year < 1900)
		t1.tm_year += 1900;
		
	if(t1.tm_year < 1970 || t1.tm_year > 2038)
		return FALSE;
	if(t1.tm_mday < 1 || t1.tm_mday > 31)
		return FALSE;
	if(t1.tm_mon < 1 || t1.tm_mon > 12)
		return FALSE;	
			
	CTime tempTime(t1.tm_year, t1.tm_mon, t1.tm_mday, t1.tm_hour, t1.tm_min, 0);
	
	*pTime = tempTime;
	
	return TRUE;
}

void ReplaceStr(CString& strText, LPCSTR szSearch, LPCSTR szReplace)
{
	
	int nPos = -1;
		
	if(0 == strText.GetLength())
		return;
		
	if(-1== (nPos = strText.Find(szSearch)))
		return;			
	
	CString strTemp = "";
	CString strReturn = "";
	
	int nSearchLen = lstrlen(szSearch);
	int nReplaceLen = lstrlen(szReplace);
	
	
	while(nPos >= 0){
		
		strTemp = strText.Mid(0, nPos);
				
		strReturn += strTemp;
		
		strTemp = strText.Mid(nPos + nSearchLen);
		strText = strTemp;
		
		strReturn += szReplace;
								
		nPos = strText.Find(szSearch);
		
		if(-1 == nPos)
			strReturn += strText;
		
	}		
		
	strText = strReturn;
			
	return;

}



