#include "stdafx.h"
#include "winfile.h"
#include <string.h>

//helper class for reading text files
//using Windows 3.1 _lread, _lopen (OpenFile()), _lclose functions
//this allows a .DLL to use more than the 15 file handles allowed to any given task


//Every Windows application is given enough memory when it is loaded to access 20 file handles. This 
//default setting is the same as the setting used in the C run-time libraries. Similarly, the first 5 handles are 
//reserved for basic file I/O, as with any other MS-DOS application, leaving the other 15 available for the 
//Windows application to use.

//however, in a .DLL, which has no task, the number of files available to the run-time functions (such as the iostream classes)
//is limited to the number still available to the .EXE caller, 

//this class allows for buffered READING of files similarly to fgets() and is used as an solution for this problem while reading
//the SQL statements from the mailed files.


IMPLEMENT_DYNAMIC(CWinFileException, CException)

CWinFileException::CWinFileException()
{

}

CWinFileException::~CWinFileException()
{

}

void AFXAPI ThrowWinFileException(int nErrCode)
{
	

	TRY
	{
		CWinFileException* pException = new CWinFileException;
		pException->m_nErrCode = nErrCode;	
		THROW(pException);
	}
	CATCH_ALL(e)
	{
		THROW_LAST();	
	}
	END_CATCH_ALL

}


IMPLEMENT_DYNAMIC(CWinFile, CObject)

CWinFile::CWinFile(UINT nBufSize /*= 4096*/)
{
	m_hFile = HFILE_ERROR;  	
	m_pszBuf = NULL;
	
	if(nBufSize < 4096)
		m_nBufSize = 4096;
	else
		m_nBufSize = nBufSize;
	
	TRY
	{
		m_pszBuf = new char[m_nBufSize];
	}
	CATCH(CMemoryException, e)
	{
		THROW_LAST();	
	}
	END_CATCH
	
	Reset();
	
}

CWinFile::~CWinFile()
{
	if(m_pszBuf)
		delete m_pszBuf;
	
	m_pszBuf = NULL;
}

void CWinFile::Reset()
{
	m_nCurPos = 0;
  	m_nEOF = m_nBufSize;
  	memset(m_pszBuf, 0, m_nBufSize);
}

void CWinFile::SeekToBegin()
{
	ASSERT_VALID(this);
	
	Reset();
	
	if(IsOpen())
		_llseek(m_hFile, 0L, 0);
	
}

void CWinFile::SeekToEnd()
{
	ASSERT_VALID(this);
	
	Reset();
	
	if(IsOpen())
		_llseek(m_hFile, 0L, 2);
	
}


BOOL CWinFile::Open(LPCSTR lpcstrFileName, int nMode /*= OF_READ*/)
{
	
	OFSTRUCT of;
	memset(&of, 0, sizeof(OFSTRUCT));
	
	m_hFile = ::OpenFile(lpcstrFileName, &of, nMode);
	
	Reset();
	
	if(HFILE_ERROR == m_hFile){
				
		ThrowWinFileException(of.nErrCode);
		return FALSE;	
	}
		
	return TRUE;		
	
}

void CWinFile::Close()
{
	if(IsOpen())
		_lclose(m_hFile);
		
	Reset();
	
	m_hFile = HFILE_ERROR;
}

LPSTR CWinFile::ReadString(LPSTR lpsz, UINT nMax)
{
	ASSERT(AfxIsValidAddress(lpsz, nMax));
	
	UINT nRead = 0;
	UINT cbRead = 0;
				
	while ((UINT)nRead < nMax - 1)
	{
		
		if(m_nCurPos >= m_nEOF)
			break;
				
		if('\0' == m_pszBuf[m_nCurPos]){
			m_nCurPos = 0;
			if(HFILE_ERROR == (cbRead = _lread(m_hFile, m_pszBuf, m_nBufSize-1)))
				return NULL;
			if(cbRead < m_nBufSize-1)
				m_nEOF = cbRead;
				
			if(0 == cbRead)
				return NULL;
				
			
		}
							
		lpsz[nRead] = m_pszBuf[m_nCurPos++];
		
		if (lpsz[nRead] == '\n'){
			lpsz[nRead] = '\0';
			
			if (lpsz[nRead -1] == '\r')
				lpsz[nRead -1] = '\0';
			
			break;
		}
		
		nRead++;
	}
	
	lpsz[nRead] = '\0';
	
	return (nRead == 0 ? NULL : lpsz);	

}
